# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import asyncio
import random

# pylint: disable=import-error
from ..otel_ot_shim_tracer import MockTracer
from ..testcase import OpenTelemetryTestCase
from ..utils import get_logger, stop_loop_when

random.seed()
logger = get_logger(__name__)


class TestAsyncio(OpenTelemetryTestCase):
    def setUp(self):
        self.tracer = MockTracer()
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)

    def tearDown(self):
        self.loop.close()
        super().tearDown()

    def test_main(self):
        # Need to run within a Task, as the scope manager depends
        # on Task.current_task()
        async def main_task():
            with self.tracer.start_active_span("parent"):
                tasks = self.submit_callbacks()
                await asyncio.gather(*tasks)

        self.loop.create_task(main_task())

        stop_loop_when(
            self.loop,
            lambda: len(self.tracer.finished_spans()) >= 4,
            timeout=5.0,
        )
        self.loop.run_forever()

        spans = self.tracer.finished_spans()
        self.assertEqual(len(spans), 4)
        self.assertNamesEqual(spans, ["task", "task", "task", "parent"])

        for idx in range(3):
            self.assertSameTrace(spans[idx], spans[-1])
            self.assertIsChildOf(spans[idx], spans[-1])

    async def task(self, interval, parent_span):
        logger.info("Starting task")

        with self.tracer.scope_manager.activate(parent_span, False):
            with self.tracer.start_active_span("task"):
                await asyncio.sleep(interval)

    def submit_callbacks(self):
        parent_span = self.tracer.scope_manager.active.span
        tasks = []
        for _ in range(3):
            interval = 0.1 + random.randint(200, 500) * 0.001
            task = self.loop.create_task(self.task(interval, parent_span))
            tasks.append(task)

        return tasks



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import unittest

from opentelemetry.exporter.zipkin import json
from opentelemetry.exporter.zipkin.proto import http


class TestZipkinExporter(unittest.TestCase):
    def test_constructors(self):
        try:
            json.ZipkinExporter()
            http.ZipkinExporter()
        except Exception as exc:  # pylint: disable=broad-exception-caught
            self.assertIsNone(exc)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# pylint: disable=too-many-lines

import os
from unittest import TestCase
from unittest.mock import Mock, PropertyMock, patch

from grpc import ChannelCredentials, Compression

from opentelemetry.attributes import BoundedAttributes
from opentelemetry.exporter.otlp.proto.common._internal import (
    _encode_key_value,
)
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
    OTLPSpanExporter,
)
from opentelemetry.exporter.otlp.proto.grpc.version import __version__
from opentelemetry.proto.collector.trace.v1.trace_service_pb2 import (
    ExportTraceServiceRequest,
)
from opentelemetry.proto.common.v1.common_pb2 import (
    AnyValue,
    ArrayValue,
    KeyValue,
)
from opentelemetry.proto.common.v1.common_pb2 import (
    InstrumentationScope as PB2InstrumentationScope,
)
from opentelemetry.proto.resource.v1.resource_pb2 import (
    Resource as OTLPResource,
)
from opentelemetry.proto.trace.v1.trace_pb2 import (
    ResourceSpans,
    ScopeSpans,
    Status,
)
from opentelemetry.proto.trace.v1.trace_pb2 import Span as OTLPSpan
from opentelemetry.sdk.environment_variables import (
    OTEL_EXPORTER_OTLP_COMPRESSION,
    OTEL_EXPORTER_OTLP_TRACES_CERTIFICATE,
    OTEL_EXPORTER_OTLP_TRACES_CLIENT_CERTIFICATE,
    OTEL_EXPORTER_OTLP_TRACES_CLIENT_KEY,
    OTEL_EXPORTER_OTLP_TRACES_COMPRESSION,
    OTEL_EXPORTER_OTLP_TRACES_ENDPOINT,
    OTEL_EXPORTER_OTLP_TRACES_HEADERS,
    OTEL_EXPORTER_OTLP_TRACES_INSECURE,
    OTEL_EXPORTER_OTLP_TRACES_TIMEOUT,
)
from opentelemetry.sdk.resources import Resource as SDKResource
from opentelemetry.sdk.trace import Status as SDKStatus
from opentelemetry.sdk.trace import StatusCode as SDKStatusCode
from opentelemetry.sdk.trace import TracerProvider, _Span
from opentelemetry.sdk.trace.export import (
    SimpleSpanProcessor,
)
from opentelemetry.sdk.util.instrumentation import InstrumentationScope
from opentelemetry.test.spantestutil import (
    get_span_with_dropped_attributes_events_links,
)

THIS_DIR = os.path.dirname(__file__)


class TestOTLPSpanExporter(TestCase):
    # pylint: disable=too-many-public-methods

    def setUp(self):
        tracer_provider = TracerProvider()
        self.exporter = OTLPSpanExporter(insecure=True)
        tracer_provider.add_span_processor(SimpleSpanProcessor(self.exporter))
        self.tracer = tracer_provider.get_tracer(__name__)

        event_mock = Mock(
            **{
                "timestamp": 1591240820506462784,
                "attributes": BoundedAttributes(
                    attributes={"a": 1, "b": False}
                ),
            }
        )

        type(event_mock).name = PropertyMock(return_value="a")
        type(event_mock).dropped_attributes = PropertyMock(return_value=0)
        self.span = _Span(
            "a",
            context=Mock(
                **{
                    "trace_state": {"a": "b", "c": "d"},
                    "span_id": 10217189687419569865,
                    "trace_id": 67545097771067222548457157018666467027,
                }
            ),
            resource=SDKResource({"a": 1, "b": False}),
            parent=Mock(**{"span_id": 12345}),
            attributes=BoundedAttributes(attributes={"a": 1, "b": True}),
            events=[event_mock],
            links=[
                Mock(
                    **{
                        "context.trace_id": 1,
                        "context.span_id": 2,
                        "attributes": BoundedAttributes(
                            attributes={"a": 1, "b": False}
                        ),
                        "dropped_attributes": 0,
                        "kind": OTLPSpan.SpanKind.SPAN_KIND_INTERNAL,  # pylint: disable=no-member
                    }
                )
            ],
            instrumentation_scope=InstrumentationScope(
                name="name", version="version"
            ),
        )

        self.span2 = _Span(
            "b",
            context=Mock(
                **{
                    "trace_state": {"a": "b", "c": "d"},
                    "span_id": 10217189687419569865,
                    "trace_id": 67545097771067222548457157018666467027,
                }
            ),
            resource=SDKResource({"a": 2, "b": False}),
            parent=Mock(**{"span_id": 12345}),
            instrumentation_scope=InstrumentationScope(
                name="name", version="version"
            ),
        )

        self.span3 = _Span(
            "c",
            context=Mock(
                **{
                    "trace_state": {"a": "b", "c": "d"},
                    "span_id": 10217189687419569865,
                    "trace_id": 67545097771067222548457157018666467027,
                }
            ),
            resource=SDKResource({"a": 1, "b": False}),
            parent=Mock(**{"span_id": 12345}),
            instrumentation_scope=InstrumentationScope(
                name="name2", version="version2"
            ),
        )

        self.span.start()
        self.span.end()
        self.span2.start()
        self.span2.end()
        self.span3.start()
        self.span3.end()

    def test_exporting(self):
        # pylint: disable=protected-access
        self.assertEqual(self.exporter._exporting, "traces")

    @patch.dict(
        "os.environ",
        {
            OTEL_EXPORTER_OTLP_TRACES_ENDPOINT: "collector:4317",
            OTEL_EXPORTER_OTLP_TRACES_HEADERS: " key1=value1,KEY2 = value=2",
            OTEL_EXPORTER_OTLP_TRACES_TIMEOUT: "10",
            OTEL_EXPORTER_OTLP_TRACES_COMPRESSION: "gzip",
        },
    )
    @patch(
        "opentelemetry.exporter.otlp.proto.grpc.exporter.OTLPExporterMixin.__init__"
    )
    def test_env_variables(self, mock_exporter_mixin):
        OTLPSpanExporter()
        self.assertTrue(len(mock_exporter_mixin.call_args_list) == 1)
        _, kwargs = mock_exporter_mixin.call_args_list[0]
        self.assertEqual(kwargs["endpoint"], "collector:4317")
        self.assertEqual(kwargs["headers"], " key1=value1,KEY2 = value=2")
        self.assertEqual(kwargs["timeout"], 10)
        self.assertEqual(kwargs["compression"], Compression.Gzip)
        self.assertIsNone(kwargs["credentials"])

    @patch.dict(
        "os.environ",
        {
            OTEL_EXPORTER_OTLP_TRACES_ENDPOINT: "collector:4317",
            OTEL_EXPORTER_OTLP_TRACES_CERTIFICATE: THIS_DIR
            + "/fixtures/test.cert",
            OTEL_EXPORTER_OTLP_TRACES_CLIENT_CERTIFICATE: THIS_DIR
            + "/fixtures/test-client-cert.pem",
            OTEL_EXPORTER_OTLP_TRACES_CLIENT_KEY: THIS_DIR
            + "/fixtures/test-client-key.pem",
            OTEL_EXPORTER_OTLP_TRACES_HEADERS: " key1=value1,KEY2 = value=2",
            OTEL_EXPORTER_OTLP_TRACES_TIMEOUT: "10",
            OTEL_EXPORTER_OTLP_TRACES_COMPRESSION: "gzip",
        },
    )
    @patch(
        "opentelemetry.exporter.otlp.proto.grpc.exporter.OTLPExporterMixin.__init__"
    )
    def test_env_variables_with_client_certificates(self, mock_exporter_mixin):
        OTLPSpanExporter()

        self.assertTrue(len(mock_exporter_mixin.call_args_list) == 1)
        _, kwargs = mock_exporter_mixin.call_args_list[0]
        self.assertEqual(kwargs["endpoint"], "collector:4317")
        self.assertEqual(kwargs["headers"], " key1=value1,KEY2 = value=2")
        self.assertEqual(kwargs["timeout"], 10)
        self.assertEqual(kwargs["compression"], Compression.Gzip)
        self.assertIsNotNone(kwargs["credentials"])
        self.assertIsInstance(kwargs["credentials"], ChannelCredentials)

    @patch.dict(
        "os.environ",
        {
            OTEL_EXPORTER_OTLP_TRACES_ENDPOINT: "collector:4317",
            OTEL_EXPORTER_OTLP_TRACES_CERTIFICATE: THIS_DIR
            + "/fixtures/test.cert",
            OTEL_EXPORTER_OTLP_TRACES_HEADERS: " key1=value1,KEY2 = value=2",
            OTEL_EXPORTER_OTLP_TRACES_TIMEOUT: "10",
            OTEL_EXPORTER_OTLP_TRACES_COMPRESSION: "gzip",
        },
    )
    @patch(
        "opentelemetry.exporter.otlp.proto.grpc.exporter.OTLPExporterMixin.__init__"
    )
    @patch("logging.Logger.error")
    def test_env_variables_with_only_certificate(
        self, mock_logger_error, mock_exporter_mixin
    ):
        OTLPSpanExporter()

        self.assertTrue(len(mock_exporter_mixin.call_args_list) == 1)
        _, kwargs = mock_exporter_mixin.call_args_list[0]
        self.assertEqual(kwargs["endpoint"], "collector:4317")
        self.assertEqual(kwargs["headers"], " key1=value1,KEY2 = value=2")
        self.assertEqual(kwargs["timeout"], 10)
        self.assertEqual(kwargs["compression"], Compression.Gzip)
        self.assertIsNotNone(kwargs["credentials"])
        self.assertIsInstance(kwargs["credentials"], ChannelCredentials)

        mock_logger_error.assert_not_called()

    @patch(
        "opentelemetry.exporter.otlp.proto.grpc.exporter.ssl_channel_credentials"
    )
    @patch("opentelemetry.exporter.otlp.proto.grpc.exporter.secure_channel")
    # pylint: disable=unused-argument
    def test_no_credentials_error(self, mock_ssl_channel, mock_secure):
        OTLPSpanExporter(insecure=False)
        self.assertTrue(mock_ssl_channel.called)

    @patch.dict(
        "os.environ",
        {OTEL_EXPORTER_OTLP_TRACES_HEADERS: " key1=value1,KEY2 = VALUE=2 "},
    )
    @patch(
        "opentelemetry.exporter.otlp.proto.grpc.exporter.ssl_channel_credentials"
    )
    @patch("opentelemetry.exporter.otlp.proto.grpc.exporter.secure_channel")
    # pylint: disable=unused-argument
    def test_otlp_headers_from_env(self, mock_ssl_channel, mock_secure):
        exporter = OTLPSpanExporter()
        # pylint: disable=protected-access
        self.assertEqual(
            exporter._headers,
            (
                ("key1", "value1"),
                ("key2", "VALUE=2"),
            ),
        )
        exporter = OTLPSpanExporter(
            headers=(("key3", "value3"), ("key4", "value4"))
        )
        # pylint: disable=protected-access
        self.assertEqual(
            exporter._headers,
            (
                ("key3", "value3"),
                ("key4", "value4"),
            ),
        )
        exporter = OTLPSpanExporter(
            headers={"key5": "value5", "key6": "value6"}
        )
        # pylint: disable=protected-access
        self.assertEqual(
            exporter._headers,
            (
                ("key5", "value5"),
                ("key6", "value6"),
            ),
        )

    @patch.dict(
        "os.environ",
        {OTEL_EXPORTER_OTLP_TRACES_INSECURE: "True"},
    )
    @patch("opentelemetry.exporter.otlp.proto.grpc.exporter.insecure_channel")
    # pylint: disable=unused-argument
    def test_otlp_insecure_from_env(self, mock_insecure):
        OTLPSpanExporter()
        # pylint: disable=protected-access
        self.assertTrue(mock_insecure.called)
        self.assertEqual(
            1,
            mock_insecure.call_count,
            f"expected {mock_insecure} to be called",
        )

    # pylint: disable=no-self-use
    @patch("opentelemetry.exporter.otlp.proto.grpc.exporter.insecure_channel")
    @patch.dict("os.environ", {OTEL_EXPORTER_OTLP_COMPRESSION: "gzip"})
    def test_otlp_exporter_otlp_compression_kwarg(self, mock_insecure_channel):
        """Specifying kwarg should take precedence over env"""
        OTLPSpanExporter(insecure=True, compression=Compression.NoCompression)
        mock_insecure_channel.assert_called_once_with(
            "localhost:4317",
            compression=Compression.NoCompression,
            options=(
                (
                    "grpc.primary_user_agent",
                    "OTel-OTLP-Exporter-Python/" + __version__,
                ),
            ),
        )

    # pylint: disable=no-self-use
    @patch("opentelemetry.exporter.otlp.proto.grpc.exporter.insecure_channel")
    @patch.dict(
        "os.environ",
        {OTEL_EXPORTER_OTLP_TRACES_COMPRESSION: "gzip"},
    )
    def test_otlp_exporter_otlp_compression_precendence(
        self, mock_insecure_channel
    ):
        """OTEL_EXPORTER_OTLP_TRACES_COMPRESSION as higher priority than
        OTEL_EXPORTER_OTLP_COMPRESSION
        """
        OTLPSpanExporter(insecure=True)
        mock_insecure_channel.assert_called_once_with(
            "localhost:4317",
            compression=Compression.Gzip,
            options=(
                (
                    "grpc.primary_user_agent",
                    "OTel-OTLP-Exporter-Python/" + __version__,
                ),
            ),
        )

    # pylint: disable=no-self-use
    @patch("opentelemetry.exporter.otlp.proto.grpc.exporter.insecure_channel")
    def test_otlp_exporter_otlp_channel_options_kwarg(
        self, mock_insecure_channel
    ):
        OTLPSpanExporter(insecure=True, channel_options=(("some", "options"),))
        mock_insecure_channel.assert_called_once_with(
            "localhost:4317",
            compression=Compression.NoCompression,
            options=(
                (
                    "grpc.primary_user_agent",
                    "OTel-OTLP-Exporter-Python/" + __version__,
                ),
                ("some", "options"),
            ),
        )

    def test_translate_spans(self):
        expected = ExportTraceServiceRequest(
            resource_spans=[
                ResourceSpans(
                    resource=OTLPResource(
                        attributes=[
                            KeyValue(key="a", value=AnyValue(int_value=1)),
                            KeyValue(
                                key="b", value=AnyValue(bool_value=False)
                            ),
                        ]
                    ),
                    scope_spans=[
                        ScopeSpans(
                            scope=PB2InstrumentationScope(
                                name="name", version="version"
                            ),
                            spans=[
                                OTLPSpan(
                                    # pylint: disable=no-member
                                    name="a",
                                    start_time_unix_nano=self.span.start_time,
                                    end_time_unix_nano=self.span.end_time,
                                    trace_state="a=b,c=d",
                                    span_id=int.to_bytes(
                                        10217189687419569865, 8, "big"
                                    ),
                                    trace_id=int.to_bytes(
                                        67545097771067222548457157018666467027,
                                        16,
                                        "big",
                                    ),
                                    parent_span_id=(
                                        b"\000\000\000\000\000\00009"
                                    ),
                                    kind=(
                                        OTLPSpan.SpanKind.SPAN_KIND_INTERNAL
                                    ),
                                    attributes=[
                                        KeyValue(
                                            key="a",
                                            value=AnyValue(int_value=1),
                                        ),
                                        KeyValue(
                                            key="b",
                                            value=AnyValue(bool_value=True),
                                        ),
                                    ],
                                    events=[
                                        OTLPSpan.Event(
                                            name="a",
                                            time_unix_nano=1591240820506462784,
                                            attributes=[
                                                KeyValue(
                                                    key="a",
                                                    value=AnyValue(
                                                        int_value=1
                                                    ),
                                                ),
                                                KeyValue(
                                                    key="b",
                                                    value=AnyValue(
                                                        bool_value=False
                                                    ),
                                                ),
                                            ],
                                        )
                                    ],
                                    status=Status(code=0, message=""),
                                    links=[
                                        OTLPSpan.Link(
                                            trace_id=int.to_bytes(
                                                1, 16, "big"
                                            ),
                                            span_id=int.to_bytes(2, 8, "big"),
                                            attributes=[
                                                KeyValue(
                                                    key="a",
                                                    value=AnyValue(
                                                        int_value=1
                                                    ),
                                                ),
                                                KeyValue(
                                                    key="b",
                                                    value=AnyValue(
                                                        bool_value=False
                                                    ),
                                                ),
                                            ],
                                            flags=0x300,
                                        )
                                    ],
                                    flags=0x300,
                                )
                            ],
                        )
                    ],
                ),
            ]
        )

        # pylint: disable=protected-access
        self.assertEqual(expected, self.exporter._translate_data([self.span]))

    def test_count_spans(self):
        # pylint: disable=protected-access
        self.assertEqual(1, self.exporter._count_data([self.span]))

    def test_translate_spans_multi(self):
        expected = ExportTraceServiceRequest(
            resource_spans=[
                ResourceSpans(
                    resource=OTLPResource(
                        attributes=[
                            KeyValue(key="a", value=AnyValue(int_value=1)),
                            KeyValue(
                                key="b", value=AnyValue(bool_value=False)
                            ),
                        ]
                    ),
                    scope_spans=[
                        ScopeSpans(
                            scope=PB2InstrumentationScope(
                                name="name", version="version"
                            ),
                            spans=[
                                OTLPSpan(
                                    # pylint: disable=no-member
                                    name="a",
                                    start_time_unix_nano=self.span.start_time,
                                    end_time_unix_nano=self.span.end_time,
                                    trace_state="a=b,c=d",
                                    span_id=int.to_bytes(
                                        10217189687419569865, 8, "big"
                                    ),
                                    trace_id=int.to_bytes(
                                        67545097771067222548457157018666467027,
                                        16,
                                        "big",
                                    ),
                                    parent_span_id=(
                                        b"\000\000\000\000\000\00009"
                                    ),
                                    kind=(
                                        OTLPSpan.SpanKind.SPAN_KIND_INTERNAL
                                    ),
                                    attributes=[
                                        KeyValue(
                                            key="a",
                                            value=AnyValue(int_value=1),
                                        ),
                                        KeyValue(
                                            key="b",
                                            value=AnyValue(bool_value=True),
                                        ),
                                    ],
                                    events=[
                                        OTLPSpan.Event(
                                            name="a",
                                            time_unix_nano=1591240820506462784,
                                            attributes=[
                                                KeyValue(
                                                    key="a",
                                                    value=AnyValue(
                                                        int_value=1
                                                    ),
                                                ),
                                                KeyValue(
                                                    key="b",
                                                    value=AnyValue(
                                                        bool_value=False
                                                    ),
                                                ),
                                            ],
                                        )
                                    ],
                                    status=Status(code=0, message=""),
                                    links=[
                                        OTLPSpan.Link(
                                            trace_id=int.to_bytes(
                                                1, 16, "big"
                                            ),
                                            span_id=int.to_bytes(2, 8, "big"),
                                            attributes=[
                                                KeyValue(
                                                    key="a",
                                                    value=AnyValue(
                                                        int_value=1
                                                    ),
                                                ),
                                                KeyValue(
                                                    key="b",
                                                    value=AnyValue(
                                                        bool_value=False
                                                    ),
                                                ),
                                            ],
                                            flags=0x300,
                                        )
                                    ],
                                    flags=0x300,
                                )
                            ],
                        ),
                        ScopeSpans(
                            scope=PB2InstrumentationScope(
                                name="name2", version="version2"
                            ),
                            spans=[
                                OTLPSpan(
                                    # pylint: disable=no-member
                                    name="c",
                                    start_time_unix_nano=self.span3.start_time,
                                    end_time_unix_nano=self.span3.end_time,
                                    trace_state="a=b,c=d",
                                    span_id=int.to_bytes(
                                        10217189687419569865, 8, "big"
                                    ),
                                    trace_id=int.to_bytes(
                                        67545097771067222548457157018666467027,
                                        16,
                                        "big",
                                    ),
                                    parent_span_id=(
                                        b"\000\000\000\000\000\00009"
                                    ),
                                    kind=(
                                        OTLPSpan.SpanKind.SPAN_KIND_INTERNAL
                                    ),
                                    status=Status(code=0, message=""),
                                    flags=0x300,
                                )
                            ],
                        ),
                    ],
                ),
                ResourceSpans(
                    resource=OTLPResource(
                        attributes=[
                            KeyValue(key="a", value=AnyValue(int_value=2)),
                            KeyValue(
                                key="b", value=AnyValue(bool_value=False)
                            ),
                        ]
                    ),
                    scope_spans=[
                        ScopeSpans(
                            scope=PB2InstrumentationScope(
                                name="name", version="version"
                            ),
                            spans=[
                                OTLPSpan(
                                    # pylint: disable=no-member
                                    name="b",
                                    start_time_unix_nano=self.span2.start_time,
                                    end_time_unix_nano=self.span2.end_time,
                                    trace_state="a=b,c=d",
                                    span_id=int.to_bytes(
                                        10217189687419569865, 8, "big"
                                    ),
                                    trace_id=int.to_bytes(
                                        67545097771067222548457157018666467027,
                                        16,
                                        "big",
                                    ),
                                    parent_span_id=(
                                        b"\000\000\000\000\000\00009"
                                    ),
                                    kind=(
                                        OTLPSpan.SpanKind.SPAN_KIND_INTERNAL
                                    ),
                                    status=Status(code=0, message=""),
                                    flags=0x300,
                                )
                            ],
                        )
                    ],
                ),
            ]
        )

        # pylint: disable=protected-access
        self.assertEqual(
            expected,
            self.exporter._translate_data([self.span, self.span2, self.span3]),
        )

    def test_count_spans_multi(self):
        self.assertEqual(
            # pylint: disable=protected-access
            3,
            self.exporter._count_data([self.span, self.span2, self.span3]),
        )

    def _check_translated_status(
        self,
        translated: ExportTraceServiceRequest,
        code_expected: Status,
    ):
        status = translated.resource_spans[0].scope_spans[0].spans[0].status

        self.assertEqual(
            status.code,
            code_expected,
        )

    def test_span_status_translate(self):
        # pylint: disable=protected-access,no-member
        unset = SDKStatus(status_code=SDKStatusCode.UNSET)
        ok = SDKStatus(status_code=SDKStatusCode.OK)
        error = SDKStatus(status_code=SDKStatusCode.ERROR)
        unset_translated = self.exporter._translate_data(
            [_create_span_with_status(unset)]
        )
        ok_translated = self.exporter._translate_data(
            [_create_span_with_status(ok)]
        )
        error_translated = self.exporter._translate_data(
            [_create_span_with_status(error)]
        )
        self._check_translated_status(
            unset_translated,
            Status.STATUS_CODE_UNSET,
        )
        self._check_translated_status(
            ok_translated,
            Status.STATUS_CODE_OK,
        )
        self._check_translated_status(
            error_translated,
            Status.STATUS_CODE_ERROR,
        )

    # pylint:disable=no-member
    def test_translate_key_values(self):
        bool_value = _encode_key_value("bool_type", False)
        self.assertTrue(isinstance(bool_value, KeyValue))
        self.assertEqual(bool_value.key, "bool_type")
        self.assertTrue(isinstance(bool_value.value, AnyValue))
        self.assertFalse(bool_value.value.bool_value)

        str_value = _encode_key_value("str_type", "str")
        self.assertTrue(isinstance(str_value, KeyValue))
        self.assertEqual(str_value.key, "str_type")
        self.assertTrue(isinstance(str_value.value, AnyValue))
        self.assertEqual(str_value.value.string_value, "str")

        int_value = _encode_key_value("int_type", 2)
        self.assertTrue(isinstance(int_value, KeyValue))
        self.assertEqual(int_value.key, "int_type")
        self.assertTrue(isinstance(int_value.value, AnyValue))
        self.assertEqual(int_value.value.int_value, 2)

        double_value = _encode_key_value("double_type", 3.2)
        self.assertTrue(isinstance(double_value, KeyValue))
        self.assertEqual(double_value.key, "double_type")
        self.assertTrue(isinstance(double_value.value, AnyValue))
        self.assertEqual(double_value.value.double_value, 3.2)

        seq_value = _encode_key_value("seq_type", ["asd", "123"])
        self.assertTrue(isinstance(seq_value, KeyValue))
        self.assertEqual(seq_value.key, "seq_type")
        self.assertTrue(isinstance(seq_value.value, AnyValue))
        self.assertTrue(isinstance(seq_value.value.array_value, ArrayValue))

        arr_value = seq_value.value.array_value
        self.assertTrue(isinstance(arr_value.values[0], AnyValue))
        self.assertEqual(arr_value.values[0].string_value, "asd")
        self.assertTrue(isinstance(arr_value.values[1], AnyValue))
        self.assertEqual(arr_value.values[1].string_value, "123")

    def test_dropped_values(self):
        span = get_span_with_dropped_attributes_events_links()
        # pylint:disable=protected-access
        translated = self.exporter._translate_data([span])
        self.assertEqual(
            1,
            translated.resource_spans[0]
            .scope_spans[0]
            .spans[0]
            .dropped_links_count,
        )
        self.assertEqual(
            2,
            translated.resource_spans[0]
            .scope_spans[0]
            .spans[0]
            .dropped_attributes_count,
        )
        self.assertEqual(
            3,
            translated.resource_spans[0]
            .scope_spans[0]
            .spans[0]
            .dropped_events_count,
        )
        self.assertEqual(
            2,
            translated.resource_spans[0]
            .scope_spans[0]
            .spans[0]
            .links[0]
            .dropped_attributes_count,
        )
        self.assertEqual(
            2,
            translated.resource_spans[0]
            .scope_spans[0]
            .spans[0]
            .events[0]
            .dropped_attributes_count,
        )


def _create_span_with_status(status: SDKStatus):
    span = _Span(
        "a",
        context=Mock(
            **{
                "trace_state": {"a": "b", "c": "d"},
                "span_id": 10217189687419569865,
                "trace_id": 67545097771067222548457157018666467027,
            }
        ),
        parent=Mock(**{"span_id": 12345}),
        instrumentation_scope=InstrumentationScope(
            name="name", version="version"
        ),
    )
    span.set_status(status)
    return span



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import logging
import os
from contextvars import Token
from uuid import uuid4

# pylint: disable=wrong-import-position
from opentelemetry.context.context import Context, _RuntimeContext  # noqa
from opentelemetry.context.contextvars_context import ContextVarsRuntimeContext
from opentelemetry.environment_variables import OTEL_PYTHON_CONTEXT

logger = logging.getLogger(__name__)


def _load_runtime_context() -> _RuntimeContext:
    """Initialize the RuntimeContext

    Returns:
        An instance of RuntimeContext.
    """
    configured_context = os.environ.get(OTEL_PYTHON_CONTEXT)
    if not configured_context:
        return ContextVarsRuntimeContext()

    # pylint: disable=import-outside-toplevel,no-name-in-module
    from opentelemetry.util._importlib_metadata import (  # noqa: PLC0415
        entry_points,
    )

    try:
        return next(
            iter(
                entry_points(
                    group="opentelemetry_context", name=configured_context
                )
            )
        ).load()()
    except Exception:  # pylint: disable=broad-exception-caught
        logger.exception(
            "Failed to load context: %s, falling back to contextvars_context",
            configured_context,
        )
        return ContextVarsRuntimeContext()


_RUNTIME_CONTEXT = _load_runtime_context()


def create_key(keyname: str) -> str:
    """To allow cross-cutting concern to control access to their local state,
    the RuntimeContext API provides a function which takes a keyname as input,
    and returns a unique key.
    Args:
        keyname: The key name is for debugging purposes and is not required to be unique.
    Returns:
        A unique string representing the newly created key.
    """
    return keyname + "-" + str(uuid4())


def get_value(key: str, context: Context | None = None) -> object:
    """To access the local state of a concern, the RuntimeContext API
    provides a function which takes a context and a key as input,
    and returns a value.

    Args:
        key: The key of the value to retrieve.
        context: The context from which to retrieve the value, if None, the current context is used.

    Returns:
        The value associated with the key.
    """
    return context.get(key) if context is not None else get_current().get(key)


def set_value(
    key: str, value: object, context: Context | None = None
) -> Context:
    """To record the local state of a cross-cutting concern, the
    RuntimeContext API provides a function which takes a context, a
    key, and a value as input, and returns an updated context
    which contains the new value.

    Args:
        key: The key of the entry to set.
        value: The value of the entry to set.
        context: The context to copy, if None, the current context is used.

    Returns:
        A new `Context` containing the value set.
    """
    if context is None:
        context = get_current()
    new_values = context.copy()
    new_values[key] = value
    return Context(new_values)


def get_current() -> Context:
    """To access the context associated with program execution,
    the Context API provides a function which takes no arguments
    and returns a Context.

    Returns:
        The current `Context` object.
    """
    return _RUNTIME_CONTEXT.get_current()


def attach(context: Context) -> Token[Context]:
    """Associates a Context with the caller's current execution unit. Returns
    a token that can be used to restore the previous Context.

    Args:
        context: The Context to set as current.

    Returns:
        A token that can be used with `detach` to reset the context.
    """
    return _RUNTIME_CONTEXT.attach(context)


def detach(token: Token[Context]) -> None:
    """Resets the Context associated with the caller's current execution unit
    to the value it had before attaching a specified Context.

    Args:
        token: The Token that was returned by a previous call to attach a Context.
    """
    try:
        _RUNTIME_CONTEXT.detach(token)
    except Exception:  # pylint: disable=broad-exception-caught
        logger.exception("Failed to detach context")


# FIXME This is a temporary location for the suppress instrumentation key.
# Once the decision around how to suppress instrumentation is made in the
# spec, this key should be moved accordingly.
_ON_EMIT_RECURSION_COUNT_KEY = create_key("on_emit_recursion_count")
_SUPPRESS_INSTRUMENTATION_KEY = create_key("suppress_instrumentation")
_SUPPRESS_HTTP_INSTRUMENTATION_KEY = create_key(
    "suppress_http_instrumentation"
)

__all__ = [
    "Context",
    "attach",
    "create_key",
    "detach",
    "get_current",
    "get_value",
    "set_value",
]



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

"""
This package implements `OpenTelemetry Resources
<https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/resource/sdk.md#resource-sdk>`_:

    *A Resource is an immutable representation of the entity producing
    telemetry. For example, a process producing telemetry that is running in
    a container on Kubernetes has a Pod name, it is in a namespace and
    possibly is part of a Deployment which also has a name. All three of
    these attributes can be included in the Resource.*

Resource objects are created with `Resource.create`, which accepts attributes
(key-values). Resources should NOT be created via constructor except by `ResourceDetector`
instances which can't use `Resource.create` to avoid infinite loops. Working with
`Resource` objects should only be done via the Resource API methods. Resource
attributes can also be passed at process invocation in the
:envvar:`OTEL_RESOURCE_ATTRIBUTES` environment variable. You should register
your resource with the  `opentelemetry.sdk.trace.TracerProvider` by passing
them into their constructors. The `Resource` passed to a provider is available
to the exporter, which can send on this information as it sees fit.

.. code-block:: python

    trace.set_tracer_provider(
        TracerProvider(
            resource=Resource.create({
                "service.name": "shoppingcart",
                "service.instance.id": "instance-12",
            }),
        ),
    )
    print(trace.get_tracer_provider().resource.attributes)

    {'telemetry.sdk.language': 'python',
    'telemetry.sdk.name': 'opentelemetry',
    'telemetry.sdk.version': '0.13.dev0',
    'service.name': 'shoppingcart',
    'service.instance.id': 'instance-12'}

Note that the OpenTelemetry project documents certain `"standard attributes"
<https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/resource/semantic_conventions/README.md>`_
that have prescribed semantic meanings, for example ``service.name`` in the
above example.
"""

# ResourceAttributes is deprecated
# pyright: reportDeprecated=false

import abc
import concurrent.futures
import logging
import os
import platform
import socket
import sys
import threading
import uuid
from collections.abc import Mapping, Sequence
from json import dumps
from os import environ
from types import ModuleType
from typing import cast
from urllib import parse

from opentelemetry.attributes import BoundedAttributes
from opentelemetry.sdk.environment_variables import (
    OTEL_EXPERIMENTAL_RESOURCE_DETECTORS,
    OTEL_RESOURCE_ATTRIBUTES,
    OTEL_SERVICE_NAME,
)
from opentelemetry.sdk.version import (
    __version__ as _OPENTELEMETRY_SDK_VERSION,
)
from opentelemetry.semconv.resource import ResourceAttributes
from opentelemetry.util.types import AttributeValue

psutil: ModuleType | None = None

try:
    import psutil as psutil_module

    psutil = psutil_module
except ImportError:
    pass

LabelValue = AttributeValue
Attributes = Mapping[str, LabelValue]
logger = logging.getLogger(__name__)

CLOUD_PROVIDER = ResourceAttributes.CLOUD_PROVIDER
CLOUD_ACCOUNT_ID = ResourceAttributes.CLOUD_ACCOUNT_ID
CLOUD_REGION = ResourceAttributes.CLOUD_REGION
CLOUD_AVAILABILITY_ZONE = ResourceAttributes.CLOUD_AVAILABILITY_ZONE
CONTAINER_NAME = ResourceAttributes.CONTAINER_NAME
CONTAINER_ID = ResourceAttributes.CONTAINER_ID
CONTAINER_IMAGE_NAME = ResourceAttributes.CONTAINER_IMAGE_NAME
CONTAINER_IMAGE_TAG = ResourceAttributes.CONTAINER_IMAGE_TAG
DEPLOYMENT_ENVIRONMENT = ResourceAttributes.DEPLOYMENT_ENVIRONMENT
FAAS_NAME = ResourceAttributes.FAAS_NAME
FAAS_ID = ResourceAttributes.FAAS_ID
FAAS_VERSION = ResourceAttributes.FAAS_VERSION
FAAS_INSTANCE = ResourceAttributes.FAAS_INSTANCE
HOST_NAME = ResourceAttributes.HOST_NAME
HOST_ARCH = ResourceAttributes.HOST_ARCH
HOST_TYPE = ResourceAttributes.HOST_TYPE
HOST_IMAGE_NAME = ResourceAttributes.HOST_IMAGE_NAME
HOST_IMAGE_ID = ResourceAttributes.HOST_IMAGE_ID
HOST_IMAGE_VERSION = ResourceAttributes.HOST_IMAGE_VERSION
KUBERNETES_CLUSTER_NAME = ResourceAttributes.K8S_CLUSTER_NAME
KUBERNETES_NAMESPACE_NAME = ResourceAttributes.K8S_NAMESPACE_NAME
KUBERNETES_POD_UID = ResourceAttributes.K8S_POD_UID
KUBERNETES_POD_NAME = ResourceAttributes.K8S_POD_NAME
KUBERNETES_CONTAINER_NAME = ResourceAttributes.K8S_CONTAINER_NAME
KUBERNETES_REPLICA_SET_UID = ResourceAttributes.K8S_REPLICASET_UID
KUBERNETES_REPLICA_SET_NAME = ResourceAttributes.K8S_REPLICASET_NAME
KUBERNETES_DEPLOYMENT_UID = ResourceAttributes.K8S_DEPLOYMENT_UID
KUBERNETES_DEPLOYMENT_NAME = ResourceAttributes.K8S_DEPLOYMENT_NAME
KUBERNETES_STATEFUL_SET_UID = ResourceAttributes.K8S_STATEFULSET_UID
KUBERNETES_STATEFUL_SET_NAME = ResourceAttributes.K8S_STATEFULSET_NAME
KUBERNETES_DAEMON_SET_UID = ResourceAttributes.K8S_DAEMONSET_UID
KUBERNETES_DAEMON_SET_NAME = ResourceAttributes.K8S_DAEMONSET_NAME
KUBERNETES_JOB_UID = ResourceAttributes.K8S_JOB_UID
KUBERNETES_JOB_NAME = ResourceAttributes.K8S_JOB_NAME
KUBERNETES_CRON_JOB_UID = ResourceAttributes.K8S_CRONJOB_UID
KUBERNETES_CRON_JOB_NAME = ResourceAttributes.K8S_CRONJOB_NAME
OS_DESCRIPTION = ResourceAttributes.OS_DESCRIPTION
OS_TYPE = ResourceAttributes.OS_TYPE
OS_VERSION = ResourceAttributes.OS_VERSION
PROCESS_PID = ResourceAttributes.PROCESS_PID
PROCESS_PARENT_PID = ResourceAttributes.PROCESS_PARENT_PID
PROCESS_EXECUTABLE_NAME = ResourceAttributes.PROCESS_EXECUTABLE_NAME
PROCESS_EXECUTABLE_PATH = ResourceAttributes.PROCESS_EXECUTABLE_PATH
PROCESS_COMMAND = ResourceAttributes.PROCESS_COMMAND
PROCESS_COMMAND_LINE = ResourceAttributes.PROCESS_COMMAND_LINE
PROCESS_COMMAND_ARGS = ResourceAttributes.PROCESS_COMMAND_ARGS
PROCESS_OWNER = ResourceAttributes.PROCESS_OWNER
PROCESS_RUNTIME_NAME = ResourceAttributes.PROCESS_RUNTIME_NAME
PROCESS_RUNTIME_VERSION = ResourceAttributes.PROCESS_RUNTIME_VERSION
PROCESS_RUNTIME_DESCRIPTION = ResourceAttributes.PROCESS_RUNTIME_DESCRIPTION
SERVICE_NAME = ResourceAttributes.SERVICE_NAME
SERVICE_NAMESPACE = ResourceAttributes.SERVICE_NAMESPACE
SERVICE_INSTANCE_ID = ResourceAttributes.SERVICE_INSTANCE_ID
SERVICE_VERSION = ResourceAttributes.SERVICE_VERSION
TELEMETRY_SDK_NAME = ResourceAttributes.TELEMETRY_SDK_NAME
TELEMETRY_SDK_VERSION = ResourceAttributes.TELEMETRY_SDK_VERSION
TELEMETRY_AUTO_VERSION = ResourceAttributes.TELEMETRY_AUTO_VERSION
TELEMETRY_SDK_LANGUAGE = ResourceAttributes.TELEMETRY_SDK_LANGUAGE


class Resource:
    """A Resource is an immutable representation of the entity producing telemetry as Attributes."""

    _attributes: BoundedAttributes
    _schema_url: str

    def __init__(self, attributes: Attributes, schema_url: str | None = None):
        self._attributes = BoundedAttributes(attributes=attributes)
        if schema_url is None:
            schema_url = ""
        self._schema_url = schema_url

    @staticmethod
    def create(
        attributes: Attributes | None = None,
        schema_url: str | None = None,
    ) -> "Resource":
        """Creates a new `Resource` from attributes.

        `ResourceDetector` instances should not call this method.

        Args:
            attributes: Optional zero or more key-value pairs.
            schema_url: Optional URL pointing to the schema

        Returns:
            The newly-created Resource.
        """

        if not attributes:
            attributes = {}

        resource = get_aggregated_resources(
            _build_resource_detectors(), _DEFAULT_RESOURCE
        ).merge(Resource(attributes, schema_url))

        if not resource.attributes.get(SERVICE_NAME, None):
            default_service_name = "unknown_service"
            process_executable_name = cast(
                str | None,
                resource.attributes.get(PROCESS_EXECUTABLE_NAME, None),
            )
            if process_executable_name:
                default_service_name += ":" + process_executable_name
            resource = resource.merge(
                Resource({SERVICE_NAME: default_service_name}, schema_url)
            )
        return resource

    @staticmethod
    def get_empty() -> "Resource":
        return _EMPTY_RESOURCE

    @property
    def attributes(self) -> Attributes:
        return self._attributes

    @property
    def schema_url(self) -> str:
        return self._schema_url

    def merge(self, other: "Resource") -> "Resource":
        """Merges this resource and an updating resource into a new `Resource`.

        If a key exists on both the old and updating resource, the value of the
        updating resource will override the old resource value.

        The updating resource's `schema_url` will be used only if the old
        `schema_url` is empty. Attempting to merge two resources with
        different, non-empty values for `schema_url` will result in an error
        and return the old resource.

        Args:
            other: The other resource to be merged.

        Returns:
            The newly-created Resource.
        """
        merged_attributes = dict(self.attributes)
        merged_attributes.update(other.attributes)

        if self.schema_url == "":
            schema_url = other.schema_url
        elif other.schema_url == "":
            schema_url = self.schema_url
        elif self.schema_url == other.schema_url:
            schema_url = other.schema_url
        else:
            logger.error(
                "Failed to merge resources: The two schemas %s and %s are incompatible",
                self.schema_url,
                other.schema_url,
            )
            return self
        return Resource(merged_attributes, schema_url)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Resource):
            return False
        return (
            self._attributes == other._attributes
            and self._schema_url == other._schema_url
        )

    def __hash__(self) -> int:
        return hash(
            f"{dumps(self._attributes.copy(), sort_keys=True)}|{self._schema_url}"
        )

    def to_json(self, indent: int | None = 4) -> str:
        return dumps(
            {
                "attributes": dict(self.attributes),
                "schema_url": self._schema_url,
            },
            indent=indent,
        )


_EMPTY_RESOURCE = Resource({})

_DEFAULT_RESOURCE = Resource(
    {
        TELEMETRY_SDK_LANGUAGE: "python",
        TELEMETRY_SDK_NAME: "opentelemetry",
        TELEMETRY_SDK_VERSION: _OPENTELEMETRY_SDK_VERSION,
    }
)


_service_instance_id: str | None = None
_service_instance_id_pid: int | None = None
_service_instance_id_lock = threading.Lock()


class ResourceDetector(abc.ABC):
    def __init__(self, raise_on_error: bool = False) -> None:
        self.raise_on_error = raise_on_error

    @abc.abstractmethod
    def detect(self) -> "Resource":
        """Don't call `Resource.create` here to avoid an infinite loop, instead instantiate `Resource` directly"""
        raise NotImplementedError()

    # pylint: disable-next=no-self-use
    def is_process_dependent(self) -> bool:
        """Return whether this detector depends on the current process identity.

        Process dependent detectors may return resource attributes that become
        stale after a process identity change, such as :func:`os.fork`.
        Detectors returning ``True`` should be re-run after such changes so the
        resulting :class:`Resource` describes the current process.

        Returns:
            ``True`` if this detector should be re-run after process identity
            changes otherwise ``False``.
        """
        return False


class OTELResourceDetector(ResourceDetector):
    # pylint: disable=no-self-use
    def detect(self) -> "Resource":
        env_resources_items = environ.get(OTEL_RESOURCE_ATTRIBUTES)
        env_resource_map: dict[str, AttributeValue] = {}

        if env_resources_items:
            for item in env_resources_items.split(","):
                try:
                    key, value = item.split("=", maxsplit=1)
                except ValueError as exc:
                    logger.warning(
                        "Invalid key value resource attribute pair %s: %s",
                        item,
                        exc,
                    )
                    continue
                value_url_decoded = parse.unquote(value.strip())
                env_resource_map[key.strip()] = value_url_decoded

        service_name = environ.get(OTEL_SERVICE_NAME)
        if service_name:
            env_resource_map[SERVICE_NAME] = service_name
        return Resource(env_resource_map)


class ProcessResourceDetector(ResourceDetector):
    """Detect process resource attributes.

    Args:
        raise_on_error: Raise errors from detection instead of logging them.
        include_command_args: Include ``process.command_args`` and
            ``process.command_line``. These attributes can contain sensitive
            command-line values, so they are excluded by default.
    """

    def __init__(
        self,
        raise_on_error: bool = False,
        *,
        include_command_args: bool = False,
    ) -> None:
        super().__init__(raise_on_error=raise_on_error)
        self._include_command_args = include_command_args

    # pylint: disable=no-self-use
    def is_process_dependent(self) -> bool:
        return True

    def detect(self) -> "Resource":
        _runtime_version = ".".join(
            map(
                str,
                (
                    sys.version_info[:3]
                    if sys.version_info.releaselevel == "final"
                    and not sys.version_info.serial
                    else sys.version_info
                ),
            )
        )
        _process_pid = os.getpid()
        _process_executable_name = sys.executable
        _process_executable_path = os.path.dirname(_process_executable_name)
        # Use sys.orig_argv, which preserves the original arguments received
        # by the interpreter. This correctly captures ``python -m <module>``
        # invocations where sys.argv is rewritten to the resolved module path
        # and the ``-m <module>`` information is lost. Only read argv[0] by
        # default because full command arguments are opt-in.
        _process_command = sys.orig_argv[0] if sys.orig_argv else ""
        resource_info: dict[str, AttributeValue] = {
            PROCESS_RUNTIME_DESCRIPTION: sys.version,
            PROCESS_RUNTIME_NAME: sys.implementation.name,
            PROCESS_RUNTIME_VERSION: _runtime_version,
            PROCESS_PID: _process_pid,
            PROCESS_EXECUTABLE_NAME: _process_executable_name,
            PROCESS_EXECUTABLE_PATH: _process_executable_path,
            PROCESS_COMMAND: _process_command,
        }
        if self._include_command_args:
            _process_argv = list(sys.orig_argv)
            resource_info[PROCESS_COMMAND_LINE] = " ".join(_process_argv)
            resource_info[PROCESS_COMMAND_ARGS] = _process_argv

        if hasattr(os, "getppid"):
            # pypy3 does not have getppid()
            resource_info[PROCESS_PARENT_PID] = os.getppid()

        if psutil is not None:
            process = psutil.Process()
            username = process.username()
            resource_info[PROCESS_OWNER] = username

        return Resource(resource_info)  # type: ignore


class OsResourceDetector(ResourceDetector):
    """Detect os resources based on `Operating System conventions <https://opentelemetry.io/docs/specs/semconv/resource/os/>`_."""

    def detect(self) -> "Resource":
        """Returns a resource with with ``os.type`` and ``os.version``.

        Python's platform library
        ~~~~~~~~~~~~~~~~~~~~~~~~~

        To grab this information, Python's ``platform`` does not return what a
        user might expect it to. Below is a breakdown of its return values in
        different operating systems.

        .. code-block:: python
            :caption: Linux

            >>> platform.system()
            'Linux'
            >>> platform.release()
            '6.5.0-35-generic'
            >>> platform.version()
            '#35~22.04.1-Ubuntu SMP PREEMPT_DYNAMIC Tue May  7 09:00:52 UTC 2'

        .. code-block:: python
            :caption: MacOS

            >>> platform.system()
            'Darwin'
            >>> platform.release()
            '23.0.0'
            >>> platform.version()
            'Darwin Kernel Version 23.0.0: Fri Sep 15 14:42:57 PDT 2023; root:xnu-10002.1.13~1/RELEASE_ARM64_T8112'

        .. code-block:: python
            :caption: Windows

            >>> platform.system()
            'Windows'
            >>> platform.release()
            '2022Server'
            >>> platform.version()
            '10.0.20348'

        .. code-block:: python
            :caption: FreeBSD

            >>> platform.system()
            'FreeBSD'
            >>> platform.release()
            '14.1-RELEASE'
            >>> platform.version()
            'FreeBSD 14.1-RELEASE releng/14.1-n267679-10e31f0946d8 GENERIC'

        .. code-block:: python
            :caption: Solaris

            >>> platform.system()
            'SunOS'
            >>> platform.release()
            '5.11'
            >>> platform.version()
            '11.4.0.15.0'

        """

        os_type = platform.system().lower()
        os_version = platform.release()

        # See docstring
        if os_type == "windows":
            os_version = platform.version()
        # Align SunOS with conventions
        elif os_type == "sunos":
            os_type = "solaris"
            os_version = platform.version()

        return Resource(
            {
                OS_TYPE: os_type,
                OS_VERSION: os_version,
            }
        )


class _HostResourceDetector(ResourceDetector):  # type: ignore[reportUnusedClass]
    """
    The HostResourceDetector detects the hostname and architecture attributes.
    """

    def detect(self) -> "Resource":
        return Resource(
            {
                HOST_NAME: socket.gethostname(),
                HOST_ARCH: platform.machine(),
            }
        )


class ServiceInstanceIdResourceDetector(ResourceDetector):
    """Detects service.instance.id as a random UUID v4.

    Per the OpenTelemetry specification, SDKs SHOULD generate a random v1/v4
    UUID for service.instance.id to uniquely identify each service instance.
    The ID is shared across all detector instances within the same process and
    regenerated automatically when the process PID changes (e.g. after a fork).

    Note: because this detector is process dependent, providers refresh it
    automatically after a fork and merge the newly detected value on top of
    the existing resource. This means that if a user explicitly sets
    `service.instance.id` (e.g. via OTEL_RESOURCE_ATTRIBUTES or
    Resource.create(attributes=...)), that value will be overwritten with a
    newly generated UUID the next time the process forks. This differs from
    Resource.create(), which does preserve user supplied overrides.
    """

    def is_process_dependent(self) -> bool:
        return True

    def detect(self) -> "Resource":
        # pylint: disable-next=global-statement
        global _service_instance_id, _service_instance_id_pid
        with _service_instance_id_lock:
            current_pid = os.getpid()
            if (
                _service_instance_id is None
                or _service_instance_id_pid != current_pid
            ):
                _service_instance_id = str(uuid.uuid4())
                _service_instance_id_pid = current_pid
            instance_id = _service_instance_id
        return Resource({SERVICE_INSTANCE_ID: instance_id})


def _build_resource_detectors() -> list["ResourceDetector"]:
    """Returns the ordered list of resource detectors to use for Resource.create.

    Fast path: if no extra detectors are configured, returns only the two
    built-in detectors without scanning entry_points.

    "service_instance" (ServiceInstanceIdResourceDetector) and "otel"
    (OTELResourceDetector) are always appended as defaults. "otel" is last so
    that OTEL_RESOURCE_ATTRIBUTES and OTEL_SERVICE_NAME take highest merge
    priority, but an explicit position in OTEL_EXPERIMENTAL_RESOURCE_DETECTORS
    is respected for either name.
    """
    detector_names: list[str] = list(
        dict.fromkeys(
            [
                name.strip()
                for name in environ.get(
                    OTEL_EXPERIMENTAL_RESOURCE_DETECTORS, ""
                ).split(",")
                if name.strip()
            ]
            + ["service_instance", "otel"]
        )
    )

    # Fast path: only the two built-in detectors — no entry_points scan needed.
    if detector_names == ["service_instance", "otel"]:
        return [ServiceInstanceIdResourceDetector(), OTELResourceDetector()]

    # pylint: disable=import-outside-toplevel
    from opentelemetry.util._importlib_metadata import (  # noqa: PLC0415
        entry_points,  # type: ignore[reportUnknownVariableType]
    )

    if "*" in detector_names:
        registered = set(
            name
            for name in entry_points(
                group="opentelemetry_resource_detector"
            ).names  # type: ignore[reportUnknownArgumentType]
            if name != "otel"
        )
        expansion = sorted(registered - set(detector_names))
        idx = detector_names.index("*")
        detector_names = (
            detector_names[:idx] + expansion + detector_names[idx + 1 :]
        )

    detectors: list[ResourceDetector] = []
    for name in detector_names:
        try:
            detectors.append(
                next(
                    iter(
                        entry_points(
                            group="opentelemetry_resource_detector",
                            name=name,
                        )  # type: ignore[reportUnknownArgumentType]
                    )
                ).load()()
            )
        except Exception:  # pylint: disable=broad-exception-caught
            logger.exception(
                "Failed to load resource detector '%s', skipping",
                name,
            )
    return detectors


def _get_process_dependent_resource() -> Resource:  # pyright: ignore[reportUnusedFunction]
    return get_aggregated_resources(
        [
            detector
            for detector in _build_resource_detectors()
            if detector.is_process_dependent()
        ],
        Resource.get_empty(),
    )


def get_aggregated_resources(
    detectors: Sequence["ResourceDetector"],
    initial_resource: Resource | None = None,
    timeout: int = 5,
) -> "Resource":
    """Retrieves resources from detectors in the order that they were passed

    :param detectors: List of resources in order of priority
    :param initial_resource: Static resource. This has highest priority
    :param timeout: Number of seconds to wait for each detector to return
    :return:
    """
    detectors_merged_resource = initial_resource or Resource.create()

    with concurrent.futures.ThreadPoolExecutor(max_workers=4) as executor:
        futures = [executor.submit(detector.detect) for detector in detectors]
        for detector_ind, future in enumerate(futures):
            detector = detectors[detector_ind]
            detected_resource: Resource = _EMPTY_RESOURCE
            try:
                detected_resource = future.result(timeout=timeout)
            except concurrent.futures.TimeoutError as ex:
                if detector.raise_on_error:
                    raise ex
                logger.warning(
                    "Detector %s took longer than %s seconds, skipping",
                    detector,
                    timeout,
                )
            # pylint: disable=broad-exception-caught
            except Exception as ex:
                if detector.raise_on_error:
                    raise ex
                logger.warning(
                    "Exception %s in detector %s, ignoring", ex, detector
                )
            finally:
                detectors_merged_resource = detectors_merged_resource.merge(
                    detected_resource
                )

    return detectors_merged_resource



# generated by datamodel-codegen:
#   filename:  schema.json
#   timestamp: 2026-06-23T15:17:37+00:00

from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import Any, ClassVar, TypeAlias

from opentelemetry.configuration._common import _additional_properties

AlwaysOffSampler: TypeAlias = dict[str, Any] | None


AlwaysOnSampler: TypeAlias = dict[str, Any] | None


@dataclass
class AttributeLimits:
    attribute_value_length_limit: int | None = None
    attribute_count_limit: int | None = None


Value: TypeAlias = list[str]


Value1: TypeAlias = list[bool]


Value2: TypeAlias = list[float]


class AttributeType(Enum):
    string = "string"
    bool = "bool"
    int = "int"
    double = "double"
    string_array = "string_array"
    bool_array = "bool_array"
    int_array = "int_array"
    double_array = "double_array"


B3MultiPropagator: TypeAlias = dict[str, Any] | None


B3Propagator: TypeAlias = dict[str, Any] | None


BaggagePropagator: TypeAlias = dict[str, Any] | None


@dataclass
class Base2ExponentialBucketHistogramAggregation:
    max_scale: int | None = None
    max_size: int | None = None
    record_min_max: bool | None = None


@dataclass
class CardinalityLimits:
    default: int | None = None
    counter: int | None = None
    gauge: int | None = None
    histogram: int | None = None
    observable_counter: int | None = None
    observable_gauge: int | None = None
    observable_up_down_counter: int | None = None
    up_down_counter: int | None = None


ConsoleExporter: TypeAlias = dict[str, Any] | None


DefaultAggregation: TypeAlias = dict[str, Any] | None


Distribution: TypeAlias = dict[str, dict[str, Any]]


DropAggregation: TypeAlias = dict[str, Any] | None


class ExemplarFilter(Enum):
    always_on = "always_on"
    always_off = "always_off"
    trace_based = "trace_based"


ExperimentalComposableAlwaysOffSampler: TypeAlias = dict[str, Any] | None


ExperimentalComposableAlwaysOnSampler: TypeAlias = dict[str, Any] | None


@dataclass
class ExperimentalComposableProbabilitySampler:
    ratio: float | None = None


@dataclass
class ExperimentalComposableRuleBasedSamplerRuleAttributePatterns:
    key: str
    included: list[str] | None = None
    excluded: list[str] | None = None


@dataclass
class ExperimentalComposableRuleBasedSamplerRuleAttributeValues:
    key: str
    values: list[str]


ExperimentalContainerResourceDetector: TypeAlias = dict[str, Any] | None


ExperimentalEventToSpanEventBridgeLogRecordProcessor: TypeAlias = (
    dict[str, Any] | None
)


ExperimentalHostResourceDetector: TypeAlias = dict[str, Any] | None


@dataclass
class ExperimentalHttpClientInstrumentation:
    request_captured_headers: list[str] | None = None
    response_captured_headers: list[str] | None = None
    known_methods: list[str] | None = None


@dataclass
class ExperimentalHttpServerInstrumentation:
    request_captured_headers: list[str] | None = None
    response_captured_headers: list[str] | None = None
    known_methods: list[str] | None = None


ExperimentalLanguageSpecificInstrumentation: TypeAlias = dict[
    str, dict[str, Any]
]


@dataclass
class ExperimentalMeterConfig:
    enabled: bool | None = None


@dataclass
class ExperimentalMeterMatcherAndConfig:
    name: str
    config: ExperimentalMeterConfig


@dataclass
class ExperimentalOtlpFileExporter:
    output_stream: str | None = None


@dataclass
class ExperimentalProbabilitySampler:
    ratio: float | None = None


ExperimentalProcessResourceDetector: TypeAlias = dict[str, Any] | None


class ExperimentalPrometheusTranslationStrategy(Enum):
    underscore_escaping_with_suffixes = "underscore_escaping_with_suffixes"
    underscore_escaping_without_suffixes_development = (
        "underscore_escaping_without_suffixes/development"
    )
    no_utf8_escaping_with_suffixes_development = (
        "no_utf8_escaping_with_suffixes/development"
    )
    no_translation_development = "no_translation/development"


@dataclass
class ExperimentalSemconvConfig:
    version: int | None = None
    experimental: bool | None = None
    dual_emit: bool | None = None


ExperimentalServiceResourceDetector: TypeAlias = dict[str, Any] | None


class ExperimentalSpanParent(Enum):
    none = "none"
    remote = "remote"
    local = "local"


@dataclass
class ExperimentalTracerConfig:
    enabled: bool | None = None


@dataclass
class ExperimentalTracerMatcherAndConfig:
    name: str
    config: ExperimentalTracerConfig


@dataclass
class ExperimentalUrlSanitization:
    sensitive_query_parameters: list[str] | None = None


@dataclass
class ExplicitBucketHistogramAggregation:
    boundaries: list[float] | None = None
    record_min_max: bool | None = None


class ExporterDefaultHistogramAggregation(Enum):
    explicit_bucket_histogram = "explicit_bucket_histogram"
    base2_exponential_bucket_histogram = "base2_exponential_bucket_histogram"


class ExporterTemporalityPreference(Enum):
    cumulative = "cumulative"
    delta = "delta"
    low_memory = "low_memory"


@dataclass
class GrpcTls:
    ca_file: str | None = None
    key_file: str | None = None
    cert_file: str | None = None
    insecure: bool | None = None


@dataclass
class HttpTls:
    ca_file: str | None = None
    key_file: str | None = None
    cert_file: str | None = None


@dataclass
class IncludeExclude:
    included: list[str] | None = None
    excluded: list[str] | None = None


class InstrumentType(Enum):
    counter = "counter"
    gauge = "gauge"
    histogram = "histogram"
    observable_counter = "observable_counter"
    observable_gauge = "observable_gauge"
    observable_up_down_counter = "observable_up_down_counter"
    up_down_counter = "up_down_counter"


LastValueAggregation: TypeAlias = dict[str, Any] | None


@dataclass
class LogRecordLimits:
    attribute_value_length_limit: int | None = None
    attribute_count_limit: int | None = None


@dataclass
class NameStringValuePair:
    name: str
    value: str | None


OpenCensusMetricProducer: TypeAlias = dict[str, Any] | None


@dataclass
class OtlpGrpcExporter:
    endpoint: str | None = None
    tls: GrpcTls | None = None
    headers: list[NameStringValuePair] | None = None
    headers_list: str | None = None
    compression: str | None = None
    timeout: int | None = None


@dataclass
class OtlpGrpcMetricExporter:
    endpoint: str | None = None
    tls: GrpcTls | None = None
    headers: list[NameStringValuePair] | None = None
    headers_list: str | None = None
    compression: str | None = None
    timeout: int | None = None
    temporality_preference: ExporterTemporalityPreference | None = None
    default_histogram_aggregation: (
        ExporterDefaultHistogramAggregation | None
    ) = None


class OtlpHttpEncoding(Enum):
    protobuf = "protobuf"
    json = "json"


@dataclass
class OtlpHttpExporter:
    endpoint: str | None = None
    tls: HttpTls | None = None
    headers: list[NameStringValuePair] | None = None
    headers_list: str | None = None
    compression: str | None = None
    timeout: int | None = None
    encoding: OtlpHttpEncoding | None = None


@dataclass
class OtlpHttpMetricExporter:
    endpoint: str | None = None
    tls: HttpTls | None = None
    headers: list[NameStringValuePair] | None = None
    headers_list: str | None = None
    compression: str | None = None
    timeout: int | None = None
    encoding: OtlpHttpEncoding | None = None
    temporality_preference: ExporterTemporalityPreference | None = None
    default_histogram_aggregation: (
        ExporterDefaultHistogramAggregation | None
    ) = None


RandomIdGenerator: TypeAlias = dict[str, Any] | None


class SeverityNumber(Enum):
    trace = "trace"
    trace2 = "trace2"
    trace3 = "trace3"
    trace4 = "trace4"
    debug = "debug"
    debug2 = "debug2"
    debug3 = "debug3"
    debug4 = "debug4"
    info = "info"
    info2 = "info2"
    info3 = "info3"
    info4 = "info4"
    warn = "warn"
    warn2 = "warn2"
    warn3 = "warn3"
    warn4 = "warn4"
    error = "error"
    error2 = "error2"
    error3 = "error3"
    error4 = "error4"
    fatal = "fatal"
    fatal2 = "fatal2"
    fatal3 = "fatal3"
    fatal4 = "fatal4"


@_additional_properties
@dataclass
class SpanExporter:
    otlp_http: OtlpHttpExporter | None = None
    otlp_grpc: OtlpGrpcExporter | None = None
    otlp_file_development: ExperimentalOtlpFileExporter | None = None
    console: ConsoleExporter | None = None
    additional_properties: ClassVar[dict[str, Any]]


class SpanKind(Enum):
    internal = "internal"
    server = "server"
    client = "client"
    producer = "producer"
    consumer = "consumer"


@dataclass
class SpanLimits:
    attribute_value_length_limit: int | None = None
    attribute_count_limit: int | None = None
    event_count_limit: int | None = None
    link_count_limit: int | None = None
    event_attribute_count_limit: int | None = None
    link_attribute_count_limit: int | None = None


SumAggregation: TypeAlias = dict[str, Any] | None


TraceContextPropagator: TypeAlias = dict[str, Any] | None


@dataclass
class TraceIdRatioBasedSampler:
    ratio: float | None = None


@dataclass
class ViewSelector:
    instrument_name: str | None = None
    instrument_type: InstrumentType | None = None
    unit: str | None = None
    meter_name: str | None = None
    meter_version: str | None = None
    meter_schema_url: str | None = None


@dataclass
class Aggregation:
    default: DefaultAggregation | None = None
    drop: DropAggregation | None = None
    explicit_bucket_histogram: ExplicitBucketHistogramAggregation | None = None
    base2_exponential_bucket_histogram: (
        Base2ExponentialBucketHistogramAggregation | None
    ) = None
    last_value: LastValueAggregation | None = None
    sum: SumAggregation | None = None


@dataclass
class AttributeNameValue:
    name: str
    value: str | float | bool | Value | Value1 | Value2 | None
    type: AttributeType | None = None


@dataclass
class BatchSpanProcessor:
    exporter: SpanExporter
    schedule_delay: int | None = None
    export_timeout: int | None = None
    max_queue_size: int | None = None
    max_export_batch_size: int | None = None


@dataclass
class ConsoleMetricExporter:
    temporality_preference: ExporterTemporalityPreference | None = None
    default_histogram_aggregation: (
        ExporterDefaultHistogramAggregation | None
    ) = None


@dataclass
class ExperimentalCodeInstrumentation:
    semconv: ExperimentalSemconvConfig | None = None


@dataclass
class ExperimentalDbInstrumentation:
    semconv: ExperimentalSemconvConfig | None = None


@dataclass
class ExperimentalGenAiInstrumentation:
    semconv: ExperimentalSemconvConfig | None = None


@dataclass
class ExperimentalHttpInstrumentation:
    semconv: ExperimentalSemconvConfig | None = None
    client: ExperimentalHttpClientInstrumentation | None = None
    server: ExperimentalHttpServerInstrumentation | None = None


@dataclass
class ExperimentalLoggerConfig:
    enabled: bool | None = None
    minimum_severity: SeverityNumber | None = None
    trace_based: bool | None = None


@dataclass
class ExperimentalLoggerMatcherAndConfig:
    name: str
    config: ExperimentalLoggerConfig


@dataclass
class ExperimentalMessagingInstrumentation:
    semconv: ExperimentalSemconvConfig | None = None


@dataclass
class ExperimentalMeterConfigurator:
    default_config: ExperimentalMeterConfig | None = None
    meters: list[ExperimentalMeterMatcherAndConfig] | None = None


@dataclass
class ExperimentalOtlpFileMetricExporter:
    output_stream: str | None = None
    temporality_preference: ExporterTemporalityPreference | None = None
    default_histogram_aggregation: (
        ExporterDefaultHistogramAggregation | None
    ) = None


@dataclass
class ExperimentalPrometheusMetricExporter:
    host: str | None = None
    port: int | None = None
    scope_info_enabled: bool | None = None
    target_info_enabled_development: bool | None = None
    resource_constant_labels: IncludeExclude | None = None
    translation_strategy: ExperimentalPrometheusTranslationStrategy | None = (
        None
    )


@_additional_properties
@dataclass
class ExperimentalResourceDetector:
    container: ExperimentalContainerResourceDetector | None = None
    host: ExperimentalHostResourceDetector | None = None
    process: ExperimentalProcessResourceDetector | None = None
    service: ExperimentalServiceResourceDetector | None = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class ExperimentalRpcInstrumentation:
    semconv: ExperimentalSemconvConfig | None = None


@dataclass
class ExperimentalSanitization:
    url: ExperimentalUrlSanitization | None = None


@dataclass
class ExperimentalTracerConfigurator:
    default_config: ExperimentalTracerConfig | None = None
    tracers: list[ExperimentalTracerMatcherAndConfig] | None = None


@_additional_properties
@dataclass
class IdGenerator:
    random: RandomIdGenerator | None = None
    additional_properties: ClassVar[dict[str, Any]]


@_additional_properties
@dataclass
class LogRecordExporter:
    otlp_http: OtlpHttpExporter | None = None
    otlp_grpc: OtlpGrpcExporter | None = None
    otlp_file_development: ExperimentalOtlpFileExporter | None = None
    console: ConsoleExporter | None = None
    additional_properties: ClassVar[dict[str, Any]]


@_additional_properties
@dataclass
class MetricProducer:
    opencensus: OpenCensusMetricProducer | None = None
    additional_properties: ClassVar[dict[str, Any]]


@_additional_properties
@dataclass
class PullMetricExporter:
    prometheus_development: ExperimentalPrometheusMetricExporter | None = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class PullMetricReader:
    exporter: PullMetricExporter
    producers: list[MetricProducer] | None = None
    cardinality_limits: CardinalityLimits | None = None


@_additional_properties
@dataclass
class PushMetricExporter:
    otlp_http: OtlpHttpMetricExporter | None = None
    otlp_grpc: OtlpGrpcMetricExporter | None = None
    otlp_file_development: ExperimentalOtlpFileMetricExporter | None = None
    console: ConsoleMetricExporter | None = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class SimpleLogRecordProcessor:
    exporter: LogRecordExporter


@dataclass
class SimpleSpanProcessor:
    exporter: SpanExporter


@_additional_properties
@dataclass
class SpanProcessor:
    batch: BatchSpanProcessor | None = None
    simple: SimpleSpanProcessor | None = None
    additional_properties: ClassVar[dict[str, Any]]


@_additional_properties
@dataclass
class TextMapPropagator:
    tracecontext: TraceContextPropagator | None = None
    baggage: BaggagePropagator | None = None
    b3: B3Propagator | None = None
    b3multi: B3MultiPropagator | None = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class ViewStream:
    name: str | None = None
    description: str | None = None
    aggregation: Aggregation | None = None
    aggregation_cardinality_limit: int | None = None
    attribute_keys: IncludeExclude | None = None


@dataclass
class BatchLogRecordProcessor:
    exporter: LogRecordExporter
    schedule_delay: int | None = None
    export_timeout: int | None = None
    max_queue_size: int | None = None
    max_export_batch_size: int | None = None


@dataclass
class ExperimentalGeneralInstrumentation:
    http: ExperimentalHttpInstrumentation | None = None
    code: ExperimentalCodeInstrumentation | None = None
    db: ExperimentalDbInstrumentation | None = None
    gen_ai: ExperimentalGenAiInstrumentation | None = None
    messaging: ExperimentalMessagingInstrumentation | None = None
    rpc: ExperimentalRpcInstrumentation | None = None
    sanitization: ExperimentalSanitization | None = None
    stability_opt_in_list: str | None = None


@dataclass
class ExperimentalInstrumentation:
    general: ExperimentalGeneralInstrumentation | None = None
    cpp: ExperimentalLanguageSpecificInstrumentation | None = None
    dotnet: ExperimentalLanguageSpecificInstrumentation | None = None
    erlang: ExperimentalLanguageSpecificInstrumentation | None = None
    go: ExperimentalLanguageSpecificInstrumentation | None = None
    java: ExperimentalLanguageSpecificInstrumentation | None = None
    js: ExperimentalLanguageSpecificInstrumentation | None = None
    php: ExperimentalLanguageSpecificInstrumentation | None = None
    python: ExperimentalLanguageSpecificInstrumentation | None = None
    ruby: ExperimentalLanguageSpecificInstrumentation | None = None
    rust: ExperimentalLanguageSpecificInstrumentation | None = None
    swift: ExperimentalLanguageSpecificInstrumentation | None = None


@dataclass
class ExperimentalLoggerConfigurator:
    default_config: ExperimentalLoggerConfig | None = None
    loggers: list[ExperimentalLoggerMatcherAndConfig] | None = None


@dataclass
class ExperimentalResourceDetection:
    attributes: IncludeExclude | None = None
    detectors: list[ExperimentalResourceDetector] | None = None


@_additional_properties
@dataclass
class LogRecordProcessor:
    batch: BatchLogRecordProcessor | None = None
    simple: SimpleLogRecordProcessor | None = None
    event_to_span_event_bridge_development: (
        ExperimentalEventToSpanEventBridgeLogRecordProcessor | None
    ) = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class PeriodicMetricReader:
    exporter: PushMetricExporter
    interval: int | None = None
    timeout: int | None = None
    max_export_batch_size_development: int | None = None
    producers: list[MetricProducer] | None = None
    cardinality_limits: CardinalityLimits | None = None


@dataclass
class Propagator:
    composite: list[TextMapPropagator] | None = None
    composite_list: str | None = None


@dataclass
class Resource:
    attributes: list[AttributeNameValue] | None = None
    detection_development: ExperimentalResourceDetection | None = None
    schema_url: str | None = None
    attributes_list: str | None = None


@dataclass
class View:
    selector: ViewSelector
    stream: ViewStream


@dataclass
class LoggerProvider:
    processors: list[LogRecordProcessor]
    limits: LogRecordLimits | None = None
    logger_configurator_development: ExperimentalLoggerConfigurator | None = (
        None
    )


@dataclass
class MetricReader:
    periodic: PeriodicMetricReader | None = None
    pull: PullMetricReader | None = None


@dataclass
class MeterProvider:
    readers: list[MetricReader]
    views: list[View] | None = None
    exemplar_filter: ExemplarFilter | None = None
    meter_configurator_development: ExperimentalMeterConfigurator | None = None


@_additional_properties
@dataclass
class OpenTelemetryConfiguration:
    file_format: str
    disabled: bool | None = None
    log_level: SeverityNumber | None = None
    attribute_limits: AttributeLimits | None = None
    logger_provider: LoggerProvider | None = None
    meter_provider: MeterProvider | None = None
    propagator: Propagator | None = None
    tracer_provider: TracerProvider | None = None
    resource: Resource | None = None
    instrumentation_development: ExperimentalInstrumentation | None = None
    distribution: Distribution | None = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class ExperimentalComposableParentThresholdSampler:
    root: ExperimentalComposableSampler


@dataclass
class ExperimentalComposableRuleBasedSampler:
    rules: list[ExperimentalComposableRuleBasedSamplerRule] | None = None


@dataclass
class ExperimentalComposableRuleBasedSamplerRule:
    """
    A rule for ExperimentalComposableRuleBasedSampler. A rule can have multiple match conditions - the sampler will be applied if all match.
    If no conditions are specified, the rule matches all spans that reach it.

    """

    sampler: ExperimentalComposableSampler
    attribute_values: (
        ExperimentalComposableRuleBasedSamplerRuleAttributeValues | None
    ) = None
    attribute_patterns: (
        ExperimentalComposableRuleBasedSamplerRuleAttributePatterns | None
    ) = None
    span_kinds: list[SpanKind] | None = None
    parent: list[ExperimentalSpanParent] | None = None


@_additional_properties
@dataclass
class ExperimentalComposableSampler:
    always_off: ExperimentalComposableAlwaysOffSampler | None = None
    always_on: ExperimentalComposableAlwaysOnSampler | None = None
    parent_threshold: ExperimentalComposableParentThresholdSampler | None = (
        None
    )
    probability: ExperimentalComposableProbabilitySampler | None = None
    rule_based: ExperimentalComposableRuleBasedSampler | None = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class ExperimentalJaegerRemoteSampler:
    endpoint: str
    initial_sampler: Sampler
    interval: int | None = None


@dataclass
class ParentBasedSampler:
    root: Sampler | None = None
    remote_parent_sampled: Sampler | None = None
    remote_parent_not_sampled: Sampler | None = None
    local_parent_sampled: Sampler | None = None
    local_parent_not_sampled: Sampler | None = None


@_additional_properties
@dataclass
class Sampler:
    always_off: AlwaysOffSampler | None = None
    always_on: AlwaysOnSampler | None = None
    composite_development: ExperimentalComposableSampler | None = None
    jaeger_remote_development: ExperimentalJaegerRemoteSampler | None = None
    parent_based: ParentBasedSampler | None = None
    probability_development: ExperimentalProbabilitySampler | None = None
    trace_id_ratio_based: TraceIdRatioBasedSampler | None = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class TracerProvider:
    processors: list[SpanProcessor]
    limits: SpanLimits | None = None
    sampler: Sampler | None = None
    id_generator: IdGenerator | None = None
    tracer_configurator_development: ExperimentalTracerConfigurator | None = (
        None
    )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# pylint: disable=redefined-outer-name, invalid-name
import pytest

from opentelemetry import trace
from opentelemetry.baggage import (
    clear,
    get_all,
    get_baggage,
    remove_baggage,
    set_baggage,
)

tracer = trace.get_tracer(__name__)


@pytest.fixture(params=[10, 100, 1000, 10000])
def baggage_size(request):
    return request.param


def set_baggage_operation(size=10):
    with tracer.start_span(name="root span"):
        ctx = get_all()
        for i in range(size):
            ctx = set_baggage(f"foo{i}", f"bar{i}", context=ctx)
    return ctx


def test_set_baggage(benchmark, baggage_size):
    ctx = benchmark(set_baggage_operation, baggage_size)
    result = get_all(ctx)
    assert len(result) == baggage_size


def test_get_baggage(benchmark, baggage_size):
    ctx = set_baggage_operation(baggage_size)

    def get_baggage_operation():
        return [get_baggage(f"foo{i}", ctx) for i in range(baggage_size)]

    result = benchmark(get_baggage_operation)
    assert result == [f"bar{i}" for i in range(baggage_size)]


def test_remove_baggage(benchmark, baggage_size):
    ctx = set_baggage_operation(baggage_size)

    def remove_operation():
        tmp_ctx = ctx
        for i in range(baggage_size):
            tmp_ctx = remove_baggage(f"foo{i}", tmp_ctx)
        return tmp_ctx

    cleared_context = benchmark(remove_operation)
    result = get_all(cleared_context)
    # After removing all baggage items, it should be empty.
    assert len(result) == 0


def test_clear_baggage(benchmark, baggage_size):
    ctx = set_baggage_operation(baggage_size)

    def clear_operation():
        return clear(ctx)

    cleared_context = benchmark(clear_operation)
    result = get_all(cleared_context)
    # After clearing the baggage should be empty.
    assert len(result) == 0



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from dataclasses import dataclass

import pytest
from pytest import param as p

from opentelemetry.sdk.trace._sampling_experimental import (
    ComposableSampler,
    composable_always_off,
    composable_always_on,
    composable_parent_threshold,
    composable_traceid_ratio_based,
    composite_sampler,
)
from opentelemetry.sdk.trace._sampling_experimental._trace_state import (
    OtelTraceState,
)
from opentelemetry.sdk.trace._sampling_experimental._util import (
    INVALID_RANDOM_VALUE,
    INVALID_THRESHOLD,
)
from opentelemetry.sdk.trace.sampling import Decision
from opentelemetry.trace import (
    NonRecordingSpan,
    SpanContext,
    TraceFlags,
    TraceState,
    set_span_in_context,
)

TRACE_ID = int("00112233445566778800000000000000", 16)
SPAN_ID = int("0123456789abcdef", 16)


@dataclass
class Input:
    sampler: ComposableSampler
    sampled: bool
    threshold: int | None
    random_value: int | None


@dataclass
class Output:
    sampled: bool
    threshold: int
    random_value: int


@pytest.mark.parametrize(
    "input,output",
    (
        p(
            Input(
                sampler=composable_always_on(),
                sampled=True,
                threshold=None,
                random_value=None,
            ),
            Output(
                sampled=True, threshold=0, random_value=INVALID_RANDOM_VALUE
            ),
            id="min threshold no parent random value",
        ),
        p(
            Input(
                sampler=composable_always_on(),
                sampled=True,
                threshold=None,
                random_value=0x7F99AA40C02744,
            ),
            Output(sampled=True, threshold=0, random_value=0x7F99AA40C02744),
            id="min threshold with parent random value",
        ),
        p(
            Input(
                sampler=composable_always_off(),
                sampled=True,
                threshold=None,
                random_value=None,
            ),
            Output(
                sampled=False,
                threshold=INVALID_THRESHOLD,
                random_value=INVALID_RANDOM_VALUE,
            ),
            id="max threshold",
        ),
        p(
            Input(
                sampler=composable_parent_threshold(composable_always_on()),
                sampled=False,
                threshold=0x7F99AA40C02744,
                random_value=0x7F99AA40C02744,
            ),
            Output(
                sampled=True,
                threshold=0x7F99AA40C02744,
                random_value=0x7F99AA40C02744,
            ),
            id="parent based in consistent mode",
        ),
        p(
            Input(
                sampler=composable_parent_threshold(composable_always_on()),
                sampled=True,
                threshold=None,
                random_value=None,
            ),
            Output(
                sampled=True,
                threshold=INVALID_THRESHOLD,
                random_value=INVALID_RANDOM_VALUE,
            ),
            id="parent based in legacy mode",
        ),
        p(
            Input(
                sampler=composable_traceid_ratio_based(0.5),
                sampled=True,
                threshold=None,
                random_value=0x7FFFFFFFFFFFFF,
            ),
            Output(
                sampled=False,
                threshold=INVALID_THRESHOLD,
                random_value=0x7FFFFFFFFFFFFF,
            ),
            id="half threshold not sampled",
        ),
        p(
            Input(
                sampler=composable_traceid_ratio_based(0.5),
                sampled=False,
                threshold=None,
                random_value=0x80000000000000,
            ),
            Output(
                sampled=True,
                threshold=0x80000000000000,
                random_value=0x80000000000000,
            ),
            id="half threshold sampled",
        ),
        p(
            Input(
                sampler=composable_traceid_ratio_based(1.0),
                sampled=False,
                threshold=0x80000000000000,
                random_value=0x80000000000000,
            ),
            Output(sampled=True, threshold=0, random_value=0x80000000000000),
            id="parent violating invariant",
        ),
    ),
)
def test_sample(input: Input, output: Output):
    parent_state = OtelTraceState.invalid()
    if input.threshold is not None:
        parent_state.threshold = input.threshold
    if input.random_value is not None:
        parent_state.random_value = input.random_value
    parent_state_str = parent_state.serialize()
    parent_trace_state = (
        TraceState((("ot", parent_state_str),)) if parent_state_str else None
    )
    flags = (
        TraceFlags(TraceFlags.SAMPLED)
        if input.sampled
        else TraceFlags.get_default()
    )
    parent_span_context = SpanContext(
        TRACE_ID, SPAN_ID, False, flags, parent_trace_state
    )
    parent_span = NonRecordingSpan(parent_span_context)
    parent_context = set_span_in_context(parent_span)

    result = composite_sampler(input.sampler).should_sample(
        parent_context, TRACE_ID, "name", trace_state=parent_trace_state
    )

    decision = Decision.RECORD_AND_SAMPLE if output.sampled else Decision.DROP
    state = OtelTraceState.parse(result.trace_state)

    assert result.decision == decision
    assert state.threshold == output.threshold
    assert state.random_value == output.random_value



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from typing import Final

ORACLE_CLOUD_REALM: Final = "oracle_cloud.realm"
"""
The OCI realm identifier that indicates the isolated partition in which the tenancy and its resources reside.
Note: See [OCI documentation on realms](https://docs.oracle.com/iaas/Content/General/Concepts/regions.htm).
"""



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase

from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics._internal.exemplar import (
    AlwaysOffExemplarFilter,
    AlwaysOnExemplarFilter,
)
from opentelemetry.sdk.metrics.export import InMemoryMetricReader
from opentelemetry.sdk.resources import SERVICE_NAME, Resource


class TestHistogramExport(TestCase):
    def test_histogram_counter_collection(self):
        in_memory_metric_reader = InMemoryMetricReader()

        provider = MeterProvider(
            resource=Resource.create({SERVICE_NAME: "otel-test"}),
            metric_readers=[in_memory_metric_reader],
        )

        meter = provider.get_meter("my-meter")

        histogram = meter.create_histogram("my_histogram")
        counter = meter.create_counter("my_counter")
        histogram.record(5, {"attribute": "value"})
        counter.add(1, {"attribute": "value_counter"})

        metric_data = in_memory_metric_reader.get_metrics_data()

        self.assertEqual(
            len(metric_data.resource_metrics[0].scope_metrics[0].metrics), 2
        )

        self.assertEqual(
            (
                metric_data.resource_metrics[0]
                .scope_metrics[0]
                .metrics[0]
                .data.data_points[0]
                .bucket_counts
            ),
            (0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
        )
        self.assertEqual(
            (
                metric_data.resource_metrics[0]
                .scope_metrics[0]
                .metrics[1]
                .data.data_points[0]
                .value
            ),
            1,
        )

        metric_data = in_memory_metric_reader.get_metrics_data()

        self.assertEqual(
            len(metric_data.resource_metrics[0].scope_metrics[0].metrics), 2
        )
        self.assertEqual(
            (
                metric_data.resource_metrics[0]
                .scope_metrics[0]
                .metrics[0]
                .data.data_points[0]
                .bucket_counts
            ),
            (0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
        )
        self.assertEqual(
            (
                metric_data.resource_metrics[0]
                .scope_metrics[0]
                .metrics[1]
                .data.data_points[0]
                .value
            ),
            1,
        )

    def test_histogram_with_exemplars(self):
        in_memory_metric_reader = InMemoryMetricReader()

        provider = MeterProvider(
            resource=Resource.create({SERVICE_NAME: "otel-test"}),
            metric_readers=[in_memory_metric_reader],
            exemplar_filter=AlwaysOnExemplarFilter(),
        )
        meter = provider.get_meter("my-meter")
        histogram = meter.create_histogram("my_histogram")

        histogram.record(
            2, {"attribute": "value1"}
        )  # Should go in the first bucket
        histogram.record(
            7, {"attribute": "value2"}
        )  # Should go in the second bucket
        histogram.record(
            9, {"attribute": "value2"}
        )  # Should also go in the second bucket
        histogram.record(
            15, {"attribute": "value3"}
        )  # Should go in the third bucket

        metric_data = in_memory_metric_reader.get_metrics_data()

        self.assertEqual(
            len(metric_data.resource_metrics[0].scope_metrics[0].metrics), 1
        )
        histogram_metric = (
            metric_data.resource_metrics[0].scope_metrics[0].metrics[0]
        )

        self.assertEqual(len(histogram_metric.data.data_points), 3)

        self.assertEqual(
            len(histogram_metric.data.data_points[0].exemplars), 1
        )
        self.assertEqual(
            len(histogram_metric.data.data_points[1].exemplars), 1
        )
        self.assertEqual(
            len(histogram_metric.data.data_points[2].exemplars), 1
        )

        self.assertEqual(histogram_metric.data.data_points[0].sum, 2)
        self.assertEqual(histogram_metric.data.data_points[1].sum, 16)
        self.assertEqual(histogram_metric.data.data_points[2].sum, 15)

        self.assertEqual(
            histogram_metric.data.data_points[0].exemplars[0].value, 2.0
        )
        self.assertEqual(
            histogram_metric.data.data_points[1].exemplars[0].value, 9.0
        )
        self.assertEqual(
            histogram_metric.data.data_points[2].exemplars[0].value, 15.0
        )

    def test_filter_with_exemplars(self):
        in_memory_metric_reader = InMemoryMetricReader()

        provider = MeterProvider(
            resource=Resource.create({SERVICE_NAME: "otel-test"}),
            metric_readers=[in_memory_metric_reader],
            exemplar_filter=AlwaysOffExemplarFilter(),
        )
        meter = provider.get_meter("my-meter")
        histogram = meter.create_histogram("my_histogram")

        histogram.record(
            2, {"attribute": "value1"}
        )  # Should go in the first bucket
        histogram.record(
            7, {"attribute": "value2"}
        )  # Should go in the second bucket

        metric_data = in_memory_metric_reader.get_metrics_data()

        self.assertEqual(
            len(metric_data.resource_metrics[0].scope_metrics[0].metrics), 1
        )
        histogram_metric = (
            metric_data.resource_metrics[0].scope_metrics[0].metrics[0]
        )

        self.assertEqual(len(histogram_metric.data.data_points), 2)

        self.assertEqual(
            len(histogram_metric.data.data_points[0].exemplars), 0
        )
        self.assertEqual(
            len(histogram_metric.data.data_points[1].exemplars), 0
        )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum
from typing import Final

CASSANDRA_CONSISTENCY_LEVEL: Final = "cassandra.consistency.level"
"""
The consistency level of the query. Based on consistency values from [CQL](https://docs.datastax.com/en/cassandra-oss/3.0/cassandra/dml/dmlConfigConsistency.html).
"""

CASSANDRA_COORDINATOR_DC: Final = "cassandra.coordinator.dc"
"""
The data center of the coordinating node for a query.
"""

CASSANDRA_COORDINATOR_ID: Final = "cassandra.coordinator.id"
"""
The ID of the coordinating node for a query.
"""

CASSANDRA_PAGE_SIZE: Final = "cassandra.page.size"
"""
The fetch size used for paging, i.e. how many rows will be returned at once.
"""

CASSANDRA_QUERY_IDEMPOTENT: Final = "cassandra.query.idempotent"
"""
Whether or not the query is idempotent.
"""

CASSANDRA_SPECULATIVE_EXECUTION_COUNT: Final = (
    "cassandra.speculative_execution.count"
)
"""
The number of times a query was speculatively executed. Not set or `0` if the query was not executed speculatively.
"""


class CassandraConsistencyLevelValues(Enum):
    ALL = "all"
    """All."""
    EACH_QUORUM = "each_quorum"
    """Each Quorum."""
    QUORUM = "quorum"
    """Quorum."""
    LOCAL_QUORUM = "local_quorum"
    """Local Quorum."""
    ONE = "one"
    """One."""
    TWO = "two"
    """Two."""
    THREE = "three"
    """Three."""
    LOCAL_ONE = "local_one"
    """Local One."""
    ANY = "any"
    """Any."""
    SERIAL = "serial"
    """Serial."""
    LOCAL_SERIAL = "local_serial"
    """Local Serial."""



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from typing import Final

SECURITY_RULE_CATEGORY: Final = "security_rule.category"
"""
A categorization value keyword used by the entity using the rule for detection of this event.
"""

SECURITY_RULE_DESCRIPTION: Final = "security_rule.description"
"""
The description of the rule generating the event.
"""

SECURITY_RULE_LICENSE: Final = "security_rule.license"
"""
Name of the license under which the rule used to generate this event is made available.
"""

SECURITY_RULE_NAME: Final = "security_rule.name"
"""
The name of the rule or signature generating the event.
"""

SECURITY_RULE_REFERENCE: Final = "security_rule.reference"
"""
Reference URL to additional information about the rule used to generate this event.
Note: The URL can point to the vendor’s documentation about the rule. If that’s not available, it can also be a link to a more general page describing this type of alert.
"""

SECURITY_RULE_RULESET_NAME: Final = "security_rule.ruleset.name"
"""
Name of the ruleset, policy, group, or parent category in which the rule used to generate this event is a member.
"""

SECURITY_RULE_UUID: Final = "security_rule.uuid"
"""
A rule ID that is unique within the scope of a set or group of agents, observers, or other entities using the rule for detection of this event.
"""

SECURITY_RULE_VERSION: Final = "security_rule.version"
"""
The version / revision of the rule being used for analysis.
"""



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0
import ipaddress
import json

from opentelemetry.exporter.zipkin.encoder import (
    _SCOPE_NAME_KEY,
    _SCOPE_VERSION_KEY,
    NAME_KEY,
    VERSION_KEY,
)
from opentelemetry.exporter.zipkin.node_endpoint import NodeEndpoint
from opentelemetry.exporter.zipkin.proto.http.v2 import ProtobufEncoder
from opentelemetry.exporter.zipkin.proto.http.v2.gen import zipkin_pb2
from opentelemetry.test.spantestutil import (
    get_span_with_dropped_attributes_events_links,
)
from opentelemetry.trace import SpanKind

from .common_tests import (  # pylint: disable=import-error
    TEST_SERVICE_NAME,
    CommonEncoderTestCases,
)


# pylint: disable=protected-access
class TestProtobufEncoder(CommonEncoderTestCases.CommonEncoderTest):
    @staticmethod
    def get_encoder(*args, **kwargs) -> ProtobufEncoder:
        return ProtobufEncoder(*args, **kwargs)

    def test_encode_trace_id(self):
        for trace_id in (1, 1024, 2**32, 2**64, 2**127):
            self.assertEqual(
                self.get_encoder_default()._encode_trace_id(trace_id),
                trace_id.to_bytes(length=16, byteorder="big", signed=False),
            )

    def test_encode_span_id(self):
        for span_id in (1, 1024, 2**8, 2**16, 2**32, 2**63):
            self.assertEqual(
                self.get_encoder_default()._encode_span_id(span_id),
                span_id.to_bytes(length=8, byteorder="big", signed=False),
            )

    def test_encode_local_endpoint_default(self):
        self.assertEqual(
            ProtobufEncoder()._encode_local_endpoint(NodeEndpoint()),
            zipkin_pb2.Endpoint(service_name=TEST_SERVICE_NAME),
        )

    def test_encode_local_endpoint_explicits(self):
        ipv4 = "192.168.0.1"
        ipv6 = "2001:db8::c001"
        port = 414120
        self.assertEqual(
            ProtobufEncoder()._encode_local_endpoint(
                NodeEndpoint(ipv4, ipv6, port)
            ),
            zipkin_pb2.Endpoint(
                service_name=TEST_SERVICE_NAME,
                ipv4=ipaddress.ip_address(ipv4).packed,
                ipv6=ipaddress.ip_address(ipv6).packed,
                port=port,
            ),
        )

    def test_encode(self):
        local_endpoint = zipkin_pb2.Endpoint(service_name=TEST_SERVICE_NAME)
        span_kind = ProtobufEncoder.SPAN_KIND_MAP[SpanKind.INTERNAL]

        otel_spans = self.get_exhaustive_otel_span_list()
        trace_id = ProtobufEncoder._encode_trace_id(
            otel_spans[0].context.trace_id
        )
        expected_output = zipkin_pb2.ListOfSpans(
            spans=[
                zipkin_pb2.Span(
                    trace_id=trace_id,
                    id=ProtobufEncoder._encode_span_id(
                        otel_spans[0].context.span_id
                    ),
                    name=otel_spans[0].name,
                    timestamp=ProtobufEncoder._nsec_to_usec_round(
                        otel_spans[0].start_time
                    ),
                    duration=(
                        ProtobufEncoder._nsec_to_usec_round(
                            otel_spans[0].end_time - otel_spans[0].start_time
                        )
                    ),
                    local_endpoint=local_endpoint,
                    kind=span_kind,
                    tags={
                        "key_bool": "false",
                        "key_string": "hello_world",
                        "key_float": "111.22",
                        "otel.status_code": "OK",
                    },
                    debug=True,
                    parent_id=ProtobufEncoder._encode_span_id(
                        otel_spans[0].parent.span_id
                    ),
                    annotations=[
                        zipkin_pb2.Annotation(
                            timestamp=ProtobufEncoder._nsec_to_usec_round(
                                otel_spans[0].events[0].timestamp
                            ),
                            value=json.dumps(
                                {
                                    "event0": {
                                        "annotation_bool": True,
                                        "annotation_string": "annotation_test",
                                        "key_float": 0.3,
                                    }
                                },
                                sort_keys=True,
                            ),
                        ),
                    ],
                ),
                zipkin_pb2.Span(
                    trace_id=trace_id,
                    id=ProtobufEncoder._encode_span_id(
                        otel_spans[1].context.span_id
                    ),
                    name=otel_spans[1].name,
                    timestamp=ProtobufEncoder._nsec_to_usec_round(
                        otel_spans[1].start_time
                    ),
                    duration=(
                        ProtobufEncoder._nsec_to_usec_round(
                            otel_spans[1].end_time - otel_spans[1].start_time
                        )
                    ),
                    local_endpoint=local_endpoint,
                    kind=span_kind,
                    tags={
                        "key_resource": "some_resource",
                        "otel.status_code": "ERROR",
                        "error": "Example description",
                    },
                    debug=False,
                ),
                zipkin_pb2.Span(
                    trace_id=trace_id,
                    id=ProtobufEncoder._encode_span_id(
                        otel_spans[2].context.span_id
                    ),
                    name=otel_spans[2].name,
                    timestamp=ProtobufEncoder._nsec_to_usec_round(
                        otel_spans[2].start_time
                    ),
                    duration=(
                        ProtobufEncoder._nsec_to_usec_round(
                            otel_spans[2].end_time - otel_spans[2].start_time
                        )
                    ),
                    local_endpoint=local_endpoint,
                    kind=span_kind,
                    tags={
                        "key_string": "hello_world",
                        "key_resource": "some_resource",
                    },
                    debug=False,
                ),
                zipkin_pb2.Span(
                    trace_id=trace_id,
                    id=ProtobufEncoder._encode_span_id(
                        otel_spans[3].context.span_id
                    ),
                    name=otel_spans[3].name,
                    timestamp=ProtobufEncoder._nsec_to_usec_round(
                        otel_spans[3].start_time
                    ),
                    duration=(
                        ProtobufEncoder._nsec_to_usec_round(
                            otel_spans[3].end_time - otel_spans[3].start_time
                        )
                    ),
                    local_endpoint=local_endpoint,
                    kind=span_kind,
                    tags={
                        NAME_KEY: "name",
                        VERSION_KEY: "version",
                        _SCOPE_NAME_KEY: "name",
                        _SCOPE_VERSION_KEY: "version",
                    },
                    debug=False,
                ),
            ],
        )

        actual_output = zipkin_pb2.ListOfSpans.FromString(
            ProtobufEncoder().serialize(otel_spans, NodeEndpoint())
        )

        self.assertEqual(actual_output, expected_output)

    def _test_encode_max_tag_length(self, max_tag_value_length: int):
        otel_span, expected_tag_output = self.get_data_for_max_tag_length_test(
            max_tag_value_length
        )
        service_name = otel_span.name

        expected_output = zipkin_pb2.ListOfSpans(
            spans=[
                zipkin_pb2.Span(
                    trace_id=ProtobufEncoder._encode_trace_id(
                        otel_span.context.trace_id
                    ),
                    id=ProtobufEncoder._encode_span_id(
                        otel_span.context.span_id
                    ),
                    name=service_name,
                    timestamp=ProtobufEncoder._nsec_to_usec_round(
                        otel_span.start_time
                    ),
                    duration=ProtobufEncoder._nsec_to_usec_round(
                        otel_span.end_time - otel_span.start_time
                    ),
                    local_endpoint=zipkin_pb2.Endpoint(
                        service_name=service_name
                    ),
                    kind=ProtobufEncoder.SPAN_KIND_MAP[SpanKind.INTERNAL],
                    tags=expected_tag_output,
                    annotations=None,
                    debug=True,
                )
            ]
        )

        actual_output = zipkin_pb2.ListOfSpans.FromString(
            ProtobufEncoder(max_tag_value_length).serialize(
                [otel_span], NodeEndpoint()
            )
        )

        self.assertEqual(actual_output, expected_output)

    def test_dropped_span_attributes(self):
        otel_span = get_span_with_dropped_attributes_events_links()
        # pylint: disable=no-member
        tags = (
            ProtobufEncoder()
            ._encode_span(otel_span, zipkin_pb2.Endpoint())
            .tags
        )

        self.assertEqual("1", tags["otel.dropped_links_count"])
        self.assertEqual("2", tags["otel.dropped_attributes_count"])
        self.assertEqual("3", tags["otel.dropped_events_count"])



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import random
import time
from concurrent.futures import (  # pylint: disable=no-name-in-module
    ThreadPoolExecutor,
)

# pylint: disable=import-error
from ..otel_ot_shim_tracer import MockTracer
from ..testcase import OpenTelemetryTestCase
from ..utils import RefCount, get_logger

random.seed()
logger = get_logger(__name__)


class TestThreads(OpenTelemetryTestCase):
    def setUp(self):  # pylint: disable=invalid-name
        self.tracer = MockTracer()
        self.executor = ThreadPoolExecutor(max_workers=3)

    def test_main(self):
        try:
            scope = self.tracer.start_active_span(
                "parent", finish_on_close=False
            )
            scope.span.ref_count = RefCount(1)
            self.submit_callbacks(scope.span)
        finally:
            scope.close()
            if scope.span.ref_count.decr() == 0:
                scope.span.finish()

        self.executor.shutdown(True)

        spans = self.tracer.finished_spans()
        self.assertEqual(len(spans), 4)
        self.assertNamesEqual(spans, ["task", "task", "task", "parent"])

        for idx in range(3):
            self.assertSameTrace(spans[idx], spans[-1])
            self.assertIsChildOf(spans[idx], spans[-1])

    def task(self, interval, parent_span):
        logger.info("Starting task")

        scope = None
        try:
            scope = self.tracer.scope_manager.activate(parent_span, False)
            with self.tracer.start_active_span("task"):
                time.sleep(interval)
        finally:
            scope.close()
            if parent_span.ref_count.decr() == 0:
                parent_span.finish()

    def submit_callbacks(self, parent_span):
        for _ in range(3):
            parent_span.ref_count.incr()
            self.executor.submit(
                self.task, 0.1 + random.randint(200, 500) * 0.001, parent_span
            )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import json
import logging
import sys
import threading
from collections.abc import Callable
from os import PathLike
from typing import IO, Generic, Literal, TypeVar

_logger = logging.getLogger(__name__)

T = TypeVar("T")


class _FileExporter(Generic[T]):
    def __init__(
        self,
        encode: Callable[[T], dict | None],
        kind: Literal["spans", "logs", "metrics"],
        logger: logging.Logger | None = None,
        path: str | PathLike[str] | None = None,
        stream: IO[str] | None = None,
    ) -> None:
        if path is not None and stream is not None:
            raise ValueError("Cannot specify both 'path' and 'stream'")
        if path is not None:
            self._stream: IO[str] = open(  # pylint: disable=consider-using-with
                path, "a", encoding="utf-8"
            )
            self._owns_stream = True
        elif stream is not None:
            self._stream = stream
            self._owns_stream = False
        else:
            self._stream = sys.stdout
            self._owns_stream = False
        self._shutdown = False
        self._encode = encode
        self._kind = kind
        self._logger = logger if logger is not None else _logger
        self._lock = threading.Lock()

    def export(self, data: T) -> bool:
        if self._shutdown:
            self._logger.warning("Exporter already shutdown, ignoring call")
            return False
        try:
            encoded = self._encode(data)
            with self._lock:
                if self._stream.closed:
                    self._logger.warning(
                        "Stream is closed, ignoring %s export call", self._kind
                    )
                    return False
                if encoded is not None:
                    self._stream.write(_format_line(encoded))
                self._stream.flush()
        # pylint: disable-next=broad-exception-caught
        except Exception as error:
            self._logger.exception(
                "Failed to write %s batch to stream: %s: %s",
                self._kind,
                type(error).__name__,
                error,
            )
            return False
        return True

    def shutdown(self) -> None:
        if self._shutdown:
            self._logger.warning("Exporter already shutdown, ignoring call")
            return
        self._shutdown = True
        if self._owns_stream:
            with self._lock:
                self._stream.close()


def _format_line(entry: dict) -> str:
    return json.dumps(entry, separators=(",", ":")) + "\n"



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from flask import Flask, request

app = Flask(__name__)


@app.route("/server_request")
def server_request():
    print(request.args.get("param"))
    return "served"


if __name__ == "__main__":
    app.run(port=8082)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from opentelemetry.exporter.otlp.json.common.metrics_encoder import (
    encode_metrics,
)
from opentelemetry.sdk.metrics import Exemplar
from tests import (
    SPAN_ID,
    TIME,
    TRACE_ID,
    make_exponential_histogram,
    make_gauge,
    make_histogram,
    make_metrics_data,
    make_sum,
)


def test_benchmark_encode_sum(benchmark):
    data = make_metrics_data([make_sum()])
    benchmark(encode_metrics, data)


def test_benchmark_encode_gauge(benchmark):
    data = make_metrics_data([make_gauge()])
    benchmark(encode_metrics, data)


def test_benchmark_encode_histogram(benchmark):
    data = make_metrics_data(
        [
            make_histogram(
                exemplars=[
                    Exemplar(
                        {"sampled": "true"}, 298.0, TIME, SPAN_ID, TRACE_ID
                    ),
                ],
            )
        ]
    )
    benchmark(encode_metrics, data)


def test_benchmark_encode_exponential_histogram(benchmark):
    data = make_metrics_data([make_exponential_histogram()])
    benchmark(encode_metrics, data)


def test_benchmark_encode_mixed_metrics(benchmark):
    data = make_metrics_data(
        [
            make_sum(name="counter"),
            make_gauge(name="gauge"),
            make_histogram(
                name="histogram",
                exemplars=[
                    Exemplar(
                        {"sampled": "true"}, 298.0, TIME, SPAN_ID, TRACE_ID
                    ),
                ],
            ),
            make_exponential_histogram(name="exp_histogram"),
        ]
    )
    benchmark(encode_metrics, data)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# AUTO-GENERATED from "opentelemetry/proto/trace/v1/trace.proto"
# DO NOT EDIT MANUALLY

from __future__ import annotations

import builtins
import dataclasses
import enum
import functools
import typing

_dataclass = functools.partial(dataclasses.dataclass, slots=True)

import opentelemetry.proto_json._json_codec
import opentelemetry.proto_json.common.v1.common
import opentelemetry.proto_json.resource.v1.resource


@typing.final
class SpanFlags(enum.IntEnum):
    """
    Generated from protobuf enum SpanFlags
    """

    SPAN_FLAGS_DO_NOT_USE = 0
    SPAN_FLAGS_TRACE_FLAGS_MASK = 255
    SPAN_FLAGS_CONTEXT_HAS_IS_REMOTE_MASK = 256
    SPAN_FLAGS_CONTEXT_IS_REMOTE_MASK = 512

@typing.final
@_dataclass
class TracesData(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message TracesData
    """

    resource_spans: builtins.list[ResourceSpans] = dataclasses.field(default_factory=builtins.list)

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.resource_spans:
            _result["resourceSpans"] = opentelemetry.proto_json._json_codec.encode_repeated(self.resource_spans, lambda _v: _v.to_dict())
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "TracesData":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            TracesData instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("resourceSpans")) is not None:
            _args["resource_spans"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: ResourceSpans.from_dict(_v), "resource_spans")

        return cls(**_args)


@typing.final
@_dataclass
class ResourceSpans(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ResourceSpans
    """

    resource: opentelemetry.proto_json.resource.v1.resource.Resource | None = None
    scope_spans: builtins.list[ScopeSpans] = dataclasses.field(default_factory=builtins.list)
    schema_url: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.resource:
            _result["resource"] = self.resource.to_dict()
        if self.scope_spans:
            _result["scopeSpans"] = opentelemetry.proto_json._json_codec.encode_repeated(self.scope_spans, lambda _v: _v.to_dict())
        if self.schema_url:
            _result["schemaUrl"] = self.schema_url
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ResourceSpans":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ResourceSpans instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("resource")) is not None:
            _args["resource"] = opentelemetry.proto_json.resource.v1.resource.Resource.from_dict(_value)
        if (_value := data.get("scopeSpans")) is not None:
            _args["scope_spans"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: ScopeSpans.from_dict(_v), "scope_spans")
        if (_value := data.get("schemaUrl")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "schema_url")
            _args["schema_url"] = _value

        return cls(**_args)


@typing.final
@_dataclass
class ScopeSpans(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ScopeSpans
    """

    scope: opentelemetry.proto_json.common.v1.common.InstrumentationScope | None = None
    spans: builtins.list[Span] = dataclasses.field(default_factory=builtins.list)
    schema_url: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.scope:
            _result["scope"] = self.scope.to_dict()
        if self.spans:
            _result["spans"] = opentelemetry.proto_json._json_codec.encode_repeated(self.spans, lambda _v: _v.to_dict())
        if self.schema_url:
            _result["schemaUrl"] = self.schema_url
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ScopeSpans":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ScopeSpans instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("scope")) is not None:
            _args["scope"] = opentelemetry.proto_json.common.v1.common.InstrumentationScope.from_dict(_value)
        if (_value := data.get("spans")) is not None:
            _args["spans"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: Span.from_dict(_v), "spans")
        if (_value := data.get("schemaUrl")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "schema_url")
            _args["schema_url"] = _value

        return cls(**_args)


@typing.final
@_dataclass
class Span(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message Span
    """

    @typing.final
    class SpanKind(enum.IntEnum):
        """
        Generated from protobuf enum SpanKind
        """

        SPAN_KIND_UNSPECIFIED = 0
        SPAN_KIND_INTERNAL = 1
        SPAN_KIND_SERVER = 2
        SPAN_KIND_CLIENT = 3
        SPAN_KIND_PRODUCER = 4
        SPAN_KIND_CONSUMER = 5

    @typing.final
    @_dataclass
    class Event(opentelemetry.proto_json._json_codec.JsonMessage):
        """
        Generated from protobuf message Event
        """

        time_unix_nano: builtins.int | None = 0
        name: builtins.str | None = ""
        attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
        dropped_attributes_count: builtins.int | None = 0

        def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
            """
            Convert this message to a dictionary with lowerCamelCase keys.

            Returns:
                Dictionary representation following OTLP JSON encoding
            """
            _result = {}
            if self.time_unix_nano:
                _result["timeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.time_unix_nano)
            if self.name:
                _result["name"] = self.name
            if self.attributes:
                _result["attributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.attributes, lambda _v: _v.to_dict())
            if self.dropped_attributes_count:
                _result["droppedAttributesCount"] = self.dropped_attributes_count
            return _result

        @builtins.classmethod
        def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Span.Event":
            """
            Create from a dictionary with lowerCamelCase keys.

            Args:
                data: Dictionary representation following OTLP JSON encoding

            Returns:
                Event instance
            """
            opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
            _args = {}

            if (_value := data.get("timeUnixNano")) is not None:
                _args["time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "time_unix_nano")
            if (_value := data.get("name")) is not None:
                opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "name")
                _args["name"] = _value
            if (_value := data.get("attributes")) is not None:
                _args["attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "attributes")
            if (_value := data.get("droppedAttributesCount")) is not None:
                opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "dropped_attributes_count")
                _args["dropped_attributes_count"] = _value

            return cls(**_args)

    @typing.final
    @_dataclass
    class Link(opentelemetry.proto_json._json_codec.JsonMessage):
        """
        Generated from protobuf message Link
        """

        trace_id: builtins.bytes | None = b""
        span_id: builtins.bytes | None = b""
        trace_state: builtins.str | None = ""
        attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
        dropped_attributes_count: builtins.int | None = 0
        flags: builtins.int | None = 0

        def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
            """
            Convert this message to a dictionary with lowerCamelCase keys.

            Returns:
                Dictionary representation following OTLP JSON encoding
            """
            _result = {}
            if self.trace_id:
                _result["traceId"] = opentelemetry.proto_json._json_codec.encode_hex(self.trace_id)
            if self.span_id:
                _result["spanId"] = opentelemetry.proto_json._json_codec.encode_hex(self.span_id)
            if self.trace_state:
                _result["traceState"] = self.trace_state
            if self.attributes:
                _result["attributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.attributes, lambda _v: _v.to_dict())
            if self.dropped_attributes_count:
                _result["droppedAttributesCount"] = self.dropped_attributes_count
            if self.flags:
                _result["flags"] = self.flags
            return _result

        @builtins.classmethod
        def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Span.Link":
            """
            Create from a dictionary with lowerCamelCase keys.

            Args:
                data: Dictionary representation following OTLP JSON encoding

            Returns:
                Link instance
            """
            opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
            _args = {}

            if (_value := data.get("traceId")) is not None:
                _args["trace_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "trace_id")
            if (_value := data.get("spanId")) is not None:
                _args["span_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "span_id")
            if (_value := data.get("traceState")) is not None:
                opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "trace_state")
                _args["trace_state"] = _value
            if (_value := data.get("attributes")) is not None:
                _args["attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "attributes")
            if (_value := data.get("droppedAttributesCount")) is not None:
                opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "dropped_attributes_count")
                _args["dropped_attributes_count"] = _value
            if (_value := data.get("flags")) is not None:
                opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "flags")
                _args["flags"] = _value

            return cls(**_args)

    trace_id: builtins.bytes | None = b""
    span_id: builtins.bytes | None = b""
    trace_state: builtins.str | None = ""
    parent_span_id: builtins.bytes | None = b""
    flags: builtins.int | None = 0
    name: builtins.str | None = ""
    kind: Span.SpanKind | builtins.int | None = 0
    start_time_unix_nano: builtins.int | None = 0
    end_time_unix_nano: builtins.int | None = 0
    attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
    dropped_attributes_count: builtins.int | None = 0
    events: builtins.list[Span.Event] = dataclasses.field(default_factory=builtins.list)
    dropped_events_count: builtins.int | None = 0
    links: builtins.list[Span.Link] = dataclasses.field(default_factory=builtins.list)
    dropped_links_count: builtins.int | None = 0
    status: Status | None = None

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.trace_id:
            _result["traceId"] = opentelemetry.proto_json._json_codec.encode_hex(self.trace_id)
        if self.span_id:
            _result["spanId"] = opentelemetry.proto_json._json_codec.encode_hex(self.span_id)
        if self.trace_state:
            _result["traceState"] = self.trace_state
        if self.parent_span_id:
            _result["parentSpanId"] = opentelemetry.proto_json._json_codec.encode_hex(self.parent_span_id)
        if self.flags:
            _result["flags"] = self.flags
        if self.name:
            _result["name"] = self.name
        if self.kind:
            _result["kind"] = builtins.int(self.kind)
        if self.start_time_unix_nano:
            _result["startTimeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.start_time_unix_nano)
        if self.end_time_unix_nano:
            _result["endTimeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.end_time_unix_nano)
        if self.attributes:
            _result["attributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.attributes, lambda _v: _v.to_dict())
        if self.dropped_attributes_count:
            _result["droppedAttributesCount"] = self.dropped_attributes_count
        if self.events:
            _result["events"] = opentelemetry.proto_json._json_codec.encode_repeated(self.events, lambda _v: _v.to_dict())
        if self.dropped_events_count:
            _result["droppedEventsCount"] = self.dropped_events_count
        if self.links:
            _result["links"] = opentelemetry.proto_json._json_codec.encode_repeated(self.links, lambda _v: _v.to_dict())
        if self.dropped_links_count:
            _result["droppedLinksCount"] = self.dropped_links_count
        if self.status:
            _result["status"] = self.status.to_dict()
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Span":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            Span instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("traceId")) is not None:
            _args["trace_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "trace_id")
        if (_value := data.get("spanId")) is not None:
            _args["span_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "span_id")
        if (_value := data.get("traceState")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "trace_state")
            _args["trace_state"] = _value
        if (_value := data.get("parentSpanId")) is not None:
            _args["parent_span_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "parent_span_id")
        if (_value := data.get("flags")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "flags")
            _args["flags"] = _value
        if (_value := data.get("name")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "name")
            _args["name"] = _value
        if (_value := data.get("kind")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "kind")
            _args["kind"] = Span.SpanKind(_value)
        if (_value := data.get("startTimeUnixNano")) is not None:
            _args["start_time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "start_time_unix_nano")
        if (_value := data.get("endTimeUnixNano")) is not None:
            _args["end_time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "end_time_unix_nano")
        if (_value := data.get("attributes")) is not None:
            _args["attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "attributes")
        if (_value := data.get("droppedAttributesCount")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "dropped_attributes_count")
            _args["dropped_attributes_count"] = _value
        if (_value := data.get("events")) is not None:
            _args["events"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: Span.Event.from_dict(_v), "events")
        if (_value := data.get("droppedEventsCount")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "dropped_events_count")
            _args["dropped_events_count"] = _value
        if (_value := data.get("links")) is not None:
            _args["links"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: Span.Link.from_dict(_v), "links")
        if (_value := data.get("droppedLinksCount")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "dropped_links_count")
            _args["dropped_links_count"] = _value
        if (_value := data.get("status")) is not None:
            _args["status"] = Status.from_dict(_value)

        return cls(**_args)


@typing.final
@_dataclass
class Status(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message Status
    """

    @typing.final
    class StatusCode(enum.IntEnum):
        """
        Generated from protobuf enum StatusCode
        """

        STATUS_CODE_UNSET = 0
        STATUS_CODE_OK = 1
        STATUS_CODE_ERROR = 2

    message: builtins.str | None = ""
    code: Status.StatusCode | builtins.int | None = 0

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.message:
            _result["message"] = self.message
        if self.code:
            _result["code"] = builtins.int(self.code)
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Status":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            Status instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("message")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "message")
            _args["message"] = _value
        if (_value := data.get("code")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "code")
            _args["code"] = Status.StatusCode(_value)

        return cls(**_args)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from collections import Counter
from collections.abc import Callable
from typing import Literal, Protocol

from opentelemetry.metrics import CallbackOptions, MeterProvider, Observation
from opentelemetry.semconv._incubating.attributes.otel_attributes import (
    OTEL_COMPONENT_NAME,
    OTEL_COMPONENT_TYPE,
    OtelComponentTypeValues,
)
from opentelemetry.semconv._incubating.metrics.otel_metrics import (
    OTEL_SDK_PROCESSOR_LOG_QUEUE_SIZE,
    OTEL_SDK_PROCESSOR_SPAN_QUEUE_SIZE,
    create_otel_sdk_processor_log_processed,
    create_otel_sdk_processor_log_queue_capacity,
    create_otel_sdk_processor_span_processed,
    create_otel_sdk_processor_span_queue_capacity,
)
from opentelemetry.semconv.attributes.error_attributes import ERROR_TYPE

_component_counter = Counter()


class ProcessorMetricsT(Protocol):
    def register_queue_size(
        self, get_queue_size: Callable[[], int]
    ) -> None: ...

    def drop_items(self, count: int) -> None: ...

    def finish_items(self, count: int, error: Exception | None) -> None: ...


class NoOpProcessorMetrics:
    def register_queue_size(self, get_queue_size: Callable[[], int]) -> None:
        pass

    def drop_items(self, count: int) -> None:
        pass

    def finish_items(self, count: int, error: Exception | None) -> None:
        pass


class ProcessorMetrics:
    def __init__(
        self,
        signal: Literal["traces", "logs"],
        component_type: OtelComponentTypeValues,
        meter_provider: MeterProvider,
        *,
        capacity: int | None = None,
    ) -> None:
        self._signal = signal
        meter = meter_provider.get_meter("opentelemetry-sdk")
        self._meter = meter

        count = _component_counter[component_type.value]
        _component_counter[component_type.value] = count + 1

        self._standard_attrs = {
            OTEL_COMPONENT_TYPE: component_type.value,
            OTEL_COMPONENT_NAME: f"{component_type.value}/{count}",
        }

        self._dropped_attrs = {
            **self._standard_attrs,
            ERROR_TYPE: "queue_full",
        }

        if signal == "traces":
            create_processed = create_otel_sdk_processor_span_processed
            create_queue_capacity = (
                create_otel_sdk_processor_span_queue_capacity
            )
        else:
            create_processed = create_otel_sdk_processor_log_processed
            create_queue_capacity = (
                create_otel_sdk_processor_log_queue_capacity
            )

        self._processed = create_processed(meter)

        if capacity is not None:
            self._queue_capacity = create_queue_capacity(meter)
            self._queue_capacity.add(capacity, self._standard_attrs)

    def register_queue_size(self, get_queue_size: Callable[[], int]) -> None:
        def record_queue_size(
            _options: CallbackOptions,
        ) -> tuple[Observation]:
            return (Observation(get_queue_size(), self._standard_attrs),)

        if self._signal == "traces":
            queue_size_name = OTEL_SDK_PROCESSOR_SPAN_QUEUE_SIZE
            queue_size_description = "The number of spans in the queue of a given instance of an SDK span processor."
            queue_size_unit = "{span}"
        else:
            queue_size_name = OTEL_SDK_PROCESSOR_LOG_QUEUE_SIZE
            queue_size_description = "The number of logs in the queue of a given instance of an SDK log processor."
            queue_size_unit = "{log}"

        self._meter.create_observable_up_down_counter(
            queue_size_name,
            callbacks=(record_queue_size,),
            description=queue_size_description,
            unit=queue_size_unit,
        )

    def drop_items(self, count: int) -> None:
        self._processed.add(count, self._dropped_attrs)

    def finish_items(self, count: int, error: Exception | None) -> None:
        if not error:
            self._processed.add(count, self._standard_attrs)
            return
        attrs = {
            **self._standard_attrs,
            ERROR_TYPE: type(error).__name__,
        }
        self._processed.add(count, attrs)


def create_processor_metrics(
    signal: Literal["traces", "logs"],
    component_type: OtelComponentTypeValues,
    meter_provider: MeterProvider,
    *,
    capacity: int | None = None,
    enabled: bool,
) -> ProcessorMetricsT:
    if not enabled:
        return NoOpProcessorMetrics()

    return ProcessorMetrics(
        signal,
        component_type,
        meter_provider,
        capacity=capacity,
    )
